#include "project.h"


#ifdef USE_RINGS
void check_rings()
{
 int i = 0;

 for(i=0; i<MAX_RINGS; i++) {
     if(rings[i].active && !rings[i].delay) {
         if(rings[i].r_change) {
             rings[i].radius += rings[i].r_change;
             if(rings[i].radius <= 0.00) { init_ring(i); }
                               }

         if(rings[i].a_change) {
             rings[i].alpha += rings[i].a_change;
             if(rings[i].alpha >  1.00) { rings[i].alpha = 1.00; }
             if(rings[i].alpha <= 0.00) { init_ring(i);          }
                               }
                                            }

     // Rings can be spawned with an initial delay, preventing drawing
     // or logical changes. Use this to create concentric rings with one group of spawn calls
     //
     if(rings[i].delay) {
         rings[i].delay--;
         if(rings[i].delay < 0) { rings[i].delay = 0; }
                        }
                            }

 return;
}




// NOTE: No implicit sorting or merging. Do that yourself if you want
//
void draw_rings()
{
 int i = 0;


 for(i=0; i<MAX_RINGS; i++) {
     if(rings[i].active && !rings[i].delay) {
         DrawRing(rings[i].pos, rings[i].radius, rings[i].radius + rings[i].thickness, 0.0, 360.00, rings[i].segments, Fade(rings[i].col, rings[i].alpha));
                                            }
                            }

 return;
}




// NOTE: Clears out a ring. Use when initializing game vars
//
void init_ring(int slot)
{
 if(slot < 0 || slot >= MAX_RINGS) { return; } // report error to framework

 rings[slot].pos.x     = 0.0;
 rings[slot].pos.y     = 0.0;
 rings[slot].radius    = 0.0;
 rings[slot].r_change  = 0.0;
 rings[slot].alpha     = 1.0;
 rings[slot].a_change  = 0.0;
 rings[slot].thickness = 1.0;
 rings[slot].segments  = 32;
 rings[slot].delay     = 0;
 rings[slot].active    = 0;

 return;
}




// NOTE: Locates an inactive ring. Returns -1 on failure
//
int get_free_ring()
{
 int i = 0;

 for(i=0; i<MAX_RINGS; i++) {
     if(!rings[i].active) { return(i); }
                            }


 return(-1);
}




int spawn_ring(float x, float y, float radius, float thickness, int delay, Color col)
{

 int slot = get_free_ring();
 if(slot == -1) { return(-1); }  // Normal, non-critical failure (no resources available)

 rings[slot].pos.x     = x;
 rings[slot].pos.y     = y;
 rings[slot].radius    = radius;
 rings[slot].r_change  = 0.0;
 rings[slot].alpha     = 1.0;
 rings[slot].a_change  = 0.0;
 rings[slot].thickness = thickness;
 rings[slot].segments  = 32;
 rings[slot].delay     = delay;
 rings[slot].col       = col;
 rings[slot].active    = 1;

 return(slot);
}

#endif





#ifdef USE_EJECTA

// NOTE: Takes last calculated polygon points and splits them into ejecta
//
void spawn_chain(int sides, Vector2 one, Vector2 two, Vector2 three, Vector2 four, Vector2 five, Vector2 six, Color col)
{
 int i = 0;
 int j = 0;
 int slot        = -1;
 Vector2 slope   = { 0 };
 Vector2 poly[6] = { one, two, three, four, five, six };

 float segments = (float)EJECTA_DENSITY;

 for(i=0; i<sides; i++) { // 0, 1, 2

     slot = get_free_chain();
     if(slot == -1) { log(LOG_ALWAYS, "no free chains"); return; }

     chain[slot].active = 1;

     if(i < (sides-1)) {
         slope.x = (poly[i+1].x - poly[i].x) / segments;
         slope.y = (poly[i+1].y - poly[i].y) / segments;

         for(j=0; j<EJECTA_DENSITY; j++) {
             set_ejecta(slot, j, poly[i].x + (slope.x * j), poly[i].y + (slope.y * j), col);
                                         }

         set_ejecta(slot, EJECTA_DENSITY, poly[i+1].x, poly[i+1].y, col);
                       }

     else {
         slope.x = (poly[0].x - poly[i].x) / segments;
         slope.y = (poly[0].y - poly[i].y) / segments;

         for(j=0; j<EJECTA_DENSITY; j++) {
             set_ejecta(slot, j, poly[i].x + (slope.x * j), poly[i].y + (slope.y * j), col);
                                         }
         set_ejecta(slot, EJECTA_DENSITY, poly[0].x, poly[0].y, col);
          }

                        }

 return;
}




void set_ejecta(int c_slot, int e_slot, float x, float y, Color col)
{
 float vel   = 2.00;
 float angle = FR_2d_angle_360_f(SCREEN_W / 2.0, SCREEN_H / 2.0, x, y);

 chain[c_slot].ejecta[e_slot].angle      = angle;
 chain[c_slot].ejecta[e_slot].velocity   = vel;
 chain[c_slot].ejecta[e_slot].pos.x      = x;
 chain[c_slot].ejecta[e_slot].pos.y      = y;

 chain[c_slot].ejecta[e_slot].path       = advance(chain[c_slot].ejecta[e_slot].angle, vel);

 chain[c_slot].ejecta[e_slot].strength   = 1.0;
 chain[c_slot].ejecta[e_slot].mutability = 25.0;
 chain[c_slot].ejecta[e_slot].decay_rate = 0.01 + quick_rand_f(0.005);
 chain[c_slot].ejecta[e_slot].col        = col;

 return;
}




void check_chains()
{
 int i          = 0;
 int j          = 0;
 float rnjesus  = 0.0;
 float rnsatan  = 0.0;
 float rnbuddha = 0.0;
 bool alive     = 0;

 for(i=0; i<MAX_CHAINS; i++) {
     if(chain[i].active) {
         alive = 0;

         for(j=0; j<EJECTA_DENSITY+1; j++) {
             chain[i].ejecta[j].pos.x += chain[i].ejecta[j].path.x;
             chain[i].ejecta[j].pos.y += chain[i].ejecta[j].path.y;

             // Check mutation chance
             //
             rnjesus = quick_rand_f(100.00);
             if(rnjesus < chain[i].ejecta[j].mutability) {
                 rnsatan  = -2.5 + quick_rand_f(5.00);
                 rnbuddha = -0.1 + quick_rand_f(0.2);
                 chain[i].ejecta[j].angle    += rnsatan;
                 chain[i].ejecta[j].velocity += rnbuddha;
                 chain[i].ejecta[j].path      = advance(chain[i].ejecta[j].angle, chain[i].ejecta[j].velocity);
                                                         }


             chain[i].ejecta[j].strength -= chain[i].ejecta[j].decay_rate;
             if(chain[i].ejecta[j].strength <= 0.0) {
                 chain[i].ejecta[j].strength   = 0.0;
                 chain[i].ejecta[j].decay_rate = 0.0;
                                                    }
             else { alive = 1; }
                                           }

         if(!alive) { // No active ejecta in this chain
             chain[i].active = 0;
                    }
                         }
                             }


 return;
}





int get_free_chain()
{
 int i = 0;

 for(i=0; i<MAX_CHAINS; i++) {
     if(!chain[i].active) { return(i); }
                             }

 return(-1);
}





void init_chains()
{
 int i = 0;
 int j = 0;

 // Pump default values into chains, and disable them
 //
 for(i=0; i<MAX_CHAINS; i++) {
     chain[i].active = 0;

     for(j=0; j<EJECTA_DENSITY; j++) {
         chain[i].ejecta[j].angle      = 0;
         chain[i].ejecta[j].velocity   = 0.0;
         chain[i].ejecta[j].pos.x      = 0.0;
         chain[i].ejecta[j].pos.y      = 0.0;
         chain[i].ejecta[j].path.x     = 0.0;
         chain[i].ejecta[j].path.y     = 0.0;
         chain[i].ejecta[j].strength   = 1.0;
         chain[i].ejecta[j].mutability = 15.0;
         chain[i].ejecta[j].decay_rate = 0.005;
         chain[i].ejecta[j].col        = WHITE;
                                     }
                             }

 return;
}





void draw_chains()
{
 int i = 0;
 int j = 0;
 float line_width = 0.0;

 //BeginShaderMode(bloom);

 // Draw Ejecta
 //
EndMode2D(); /// This is a hack for Vampire pimps

 for(i=0; i<MAX_CHAINS; i++) {
     if(chain[i].active) {
         for(j=0; j<EJECTA_DENSITY; j++) {

             line_width = scale(chain[i].ejecta[j].strength, 0.0, 1.0, 2.0, 10.0);
// 10.00 * chain[i].ejecta[j].strength;

             DrawLineEx(chain[i].ejecta[j].pos, chain[i].ejecta[j+1].pos, line_width, Fade(chain[i].ejecta[j].col, chain[i].ejecta[j].strength * 0.80));
                                         }
                         }
                             }

BeginMode2D(camera); /// this is a hack for Vampire pimps

 //EndShaderMode();

 return;
}


#endif
