#include "Project.h"

#ifdef USE_MATH


// NOTE: Hyperbolic tangential scaling routine
//       Translates source to scaled, and uses a tanh()
//       to generate an increasing scale at the end.
//       USE THIS FOR DECELERATION-SPECIFIC CALCULATIONS
// 100 %                                                                 .
// S                                                                   ..
// C                                                             ......
// A                                                    .........
// L                                         ...........
// E                          ...............
// 0 %        ................
// Percentage 0-----------------------------------------------------------100
//
float tan_scale(float value, float source_min, float source_max, float scaled_min, float scaled_max)
{
 long double percentage     = 0;
 long double log_percentage = 0;

 if((source_max - source_min) == 0.000) { return(0); } // If used in framework, signal division by zero

 // Reality check
 // If value is at the max value, output scaled max value
 // Otherwise our final value can float around the max value
 //
 if(value <= source_min) { return(scaled_min); }
 if(value >= source_max) { return(scaled_max); }

 // Log(0) is undefined, so let's avoid that the ghetto way
 //
 percentage = (value - source_min) / (source_max - source_min);
 if(percentage < 0.0001) { percentage = 0.0001; }

 log_percentage = tanh(percentage * 3.0); /// Uh, where is this magic number from? Should it be variable?

 return ((log_percentage * (scaled_max - scaled_min)) + scaled_min);
}






// NOTE: Performs opposite action (accelerates)
//
float log_scale(float value, float source_min, float source_max, float scaled_min, float scaled_max)
{
 long double percentage     = 0;
 long double log_percentage = 0;

 if((source_max - source_min) == 0.000) { return(0); } // If used in framework, signal division by zero

 if(value <= source_min) { return(scaled_min); }
 if(value >= source_max) { return(scaled_max); }

 // Log(0) is undefined, so let's avoid that the ghetto way
 //
 percentage = 1.0 - ((value - source_min) / (source_max - source_min));
 if(percentage < 0.0001) { percentage = 0.0001; }

 log_percentage = 1.0 - (log10(percentage * 100.0) / 2.0);

 return ((log_percentage * (scaled_max - scaled_min)) + scaled_min);
}





// NOTE: Performs normalization of floating point values. Returns 0-1
//
float norm_f(float source, float min, float max)
{

 return((source - min) / (max - min));
}





// ------------------------------------------------------------------------------
// FRAMEWORK DOCUMENTATION
//
// NOTE - Returns the angle in degrees (0-360) between two coordinate pairs,
//        with the assumption that zero degrees points directly UP, and rotates
//        clockwise.
//
// ------------------------------------------------------------------------------
//
float FR_2d_angle_360_f(float x1, float y1, float x2, float y2)
{
 float angle = 0;
 float x_diff, y_diff;


 x_diff = (x2 - x1);
 y_diff = (y2 - y1);

 angle = atan2(y_diff, x_diff);
 angle *= (180.00 / M_PI);


 // This produces an angle where the zero point is facing dead left <---
 // So correct this by adding 90 degrees.
 //
 angle += 90.00;


 // Correct the Radians/angle snafu by adding 360 to the total if the result is negative
 // Otherwise you go from 180 to -180 in the middle of your circle. Boo! Hiss!
 //
 if(angle < 0) { angle += 360.00; }

 return(angle);
}





// NOTE: Returns a point, rotated around another point, by so many degrees
//
Vector2 rotate_point_2d(float point_x, float point_y, float around_x, float around_y, float angle)
{
  float s = sin(degrees_to_radians(angle));
  float c = cos(degrees_to_radians(angle));

  Vector2 results = { 0 };
  Vector2 xlated  = { 0 };

  // normalize point to origin
  //
  point_x -= around_x;
  point_y -= around_y;

  // Perform rotation
  //
  xlated.x = (point_x * c) - (point_y * s);
  xlated.y = (point_x * s) + (point_y * c);

  // de-translate point
  //
  results.x = xlated.x + around_x;
  results.y = xlated.y + around_y;

  return(results);
}




// NOTE: This function takes two angles and determines the shortest distance
//       to rotate between them. It will return the distance in degrees, and
//       will be positive for clockwise rotation, and negative for counter-
//       clockwise rotation.
//
// NOTE: Due to use of mod operator, the results will be integer. Watch for
//       precision loss!
//
int rotate_to_angle(int angle_a, int angle_b)
{
 int offset = 360 - angle_a;
 int b2     = (angle_b + offset) % 360;
 int dist   = 0;

 if(b2 < 180) { dist = b2; }
 else         { dist = b2 - 360; }

 return(dist);
}





// NOTE: Basic scaling routine
//       Translates source to scaled
//
float scale(float value, float source_min, float source_max, float scaled_min, float scaled_max)
{
 long double percentage = 0;

 if((source_max - source_min) == 0.000) { return(0); } // If used in framework, signal division by zero

 percentage = (value - source_min) / (source_max - source_min);

 return ((percentage * (scaled_max - scaled_min)) + scaled_min);
}






float degrees_to_radians(float degrees)
{
 float value = 0;

 value = degrees * (PI / 180.00);

 return(value);
}




float sin_march(float angle, float magnitude)
{

 return(sin(degrees_to_radians(angle)) * magnitude);

}


float cos_march(float angle, float magnitude)
{

 return(cos(degrees_to_radians(angle)) * magnitude);

}


#endif

