#include "Project.h"

#ifdef USE_MUSIC



void music_handler()
{
 int i             = 0;
 bool check        = 0;
 float time_played = 0.0;

 for(i=0; i<MAX_TRACKS; i++) {
     if(muzak[i].loaded) {
     switch(muzak[i].state) {
         case 0: // Inactive
                 check = IsMusicStreamPlaying(muzak[i].track);
                 if(check) { StopMusicStream(muzak[i].track); }
                 break;

         case 2: // Fading in
                 muzak[i].volume += 0.01;
                 if(muzak[i].volume >= 1.00) {
                     muzak[i].volume = 1.00;
                     muzak[i].state = 3;
                                             }
                 check = IsMusicStreamPlaying(muzak[i].track);
                 if(!check) { PlayMusicStream(muzak[i].track);   }

                 SetMusicVolume(muzak[i].track, muzak[i].volume * muzak[i].max_volume);
                 UpdateMusicStream(muzak[i].track);
                 break;

         case 3: // Playing
                 check = IsMusicStreamPlaying(muzak[i].track);
                 if(!check) { PlayMusicStream(muzak[i].track);   }
                 UpdateMusicStream(muzak[i].track);

                 // If track is reporting *close* to being done, stop it
                 // The time played value wraps around from 1.0 to 0.0,
                 // so you need to catch it slightly before the loop.
                 //
                 // NOTE: Look for a more reliable way to accomplish this
                 //
                 time_played = GetMusicTimePlayed(muzak[i].track) / GetMusicTimeLength(muzak[i].track);
                 ///log("time_played = %f", time_played);
                 if(time_played >= 0.985 && !muzak[i].repeat) { StopMusicStream(muzak[i].track); muzak[i].state = 0; }

                 break;

         case 4: // Pausing
                 check = IsMusicStreamPlaying(muzak[i].track);
                 if(check) { PauseMusicStream(muzak[i].track);  }
                 else      { UpdateMusicStream(muzak[i].track); }
                 break;
/*
         case 5: // Resuming
                 check = IsMusicStreamPlaying(muzak[i].track);
                 if(!check) {
                     ResumeMusicStream(muzak[i].track);
                     muzak[i].state = 3;
                            }
                 break;
*/
         case 6: // Fading out (pauses when done)
                 muzak[i].volume -= muzak[i].fade_rate;
                 if(muzak[i].volume <= 0.00) {
                     muzak[i].volume = 0.00;
                     muzak[i].state = 4;
                                             }

                 SetMusicVolume(muzak[i].track, muzak[i].volume * muzak[i].max_volume);
                 UpdateMusicStream(muzak[i].track);
                 break;

                            }
                         }
                             }

 return;
}




void load_music(int slot, const char *file)
{
 if(slot < 0 || slot >= MAX_TRACKS) { log(LOG_ALWAYS, "[ ERROR ] : Slot OOB in call to load music. Slot = %d", slot); return; } // Throw proper error, please

 #ifdef USE_IRQ_LOADER
     queue_music(&muzak[slot].track, (char *)file);
 #else
     muzak[slot].track    = LoadMusicStream(file);
     if(muzak[slot].track.ctxData == NULL) { log(LOG_ALWAYS, "[ ERROR ] : Audio file was not successfully loaded. Slot = %d", slot); return; }
 #endif // FR_USE_IRQ_LOADER

 muzak[slot].loaded     = 1;
 muzak[slot].state      = 0;
 muzak[slot].repeat     = 1;
 muzak[slot].position   = 0.0;
 muzak[slot].volume     = 1.0;
 muzak[slot].max_volume = 1.0;
 muzak[slot].pitch      = 1.0;

 return;
}






void init_music()
{
 int i = 0;

 for(i=0; i<MAX_TRACKS; i++) {
     muzak[i].loaded     = 0;
     muzak[i].state      = 0;
     muzak[i].repeat     = 1;
     muzak[i].position   = 0.0;
     muzak[i].volume     = 1.0;
     muzak[i].max_volume = 1.0;
     muzak[i].fade_rate  = 0.01;
     muzak[i].pitch      = 1.0;
                             }

 return;
}




// Begins playing music from the very beginning
//
void rewind_music(int slot)
{
 if(slot < 0 || slot >= MAX_TRACKS) { log(LOG_ALWAYS, "[ ERROR ] : Invalid track provided to %s  track = %d", __func__, slot); return; }

 SeekMusicStream(muzak[slot].track, 0.0);
 SetMusicVolume(muzak[slot].track, muzak[slot].volume * muzak[slot].max_volume);
 PlayMusicStream(muzak[slot].track);

 muzak[slot].state = 3;

 return;
}




// Starts playing music, but doesn't force a rewind,
// so after a pause you can call this to resume playing where
// you left off
//
void start_music(int slot)
{
 if(slot < 0 || slot >= MAX_TRACKS) { log(LOG_ALWAYS, "[ ERROR ] : Invalid track provided to %s  track = %d", __func__, slot); return; }

 SetMusicVolume(muzak[slot].track, muzak[slot].volume * muzak[slot].max_volume);
 PlayMusicStream(muzak[slot].track);

 muzak[slot].state = 3;

 return;
}




void fade_music_in(int slot)
{
 if(slot < 0 || slot >= MAX_TRACKS) {  log(LOG_ALWAYS, "[ ERROR ] : Invalid track provided to %s  track = %d", __func__, slot); return; }

 PlayMusicStream(muzak[slot].track);

 muzak[slot].volume = 0.0;
 muzak[slot].state  = 2;

 return;
}





void resume_music(int slot)
{
 if(slot < 0 || slot >= MAX_TRACKS) { log(LOG_ALWAYS, "[ ERROR ] : Invalid track provided to %s  track = %d", __func__, slot); return; }

 muzak[slot].state  = 2;

 return;
}





void fade_all_music()
{
 int i = 0;
 bool check = 0;

 for(i=0; i<MAX_TRACKS; i++) {
     if(muzak[i].state == 3) { muzak[i].state = 6; }
                             }

 return;
}




void fade_music_out(int slot)
{
 if(slot < 0 || slot >= MAX_TRACKS) { log(LOG_ALWAYS, "[ ERROR ] : Invalid track provided to %s  track = %d", __func__, slot); return; }

 muzak[slot].state  = 6;

 return;
}




#endif
