#include "Project.h"




Vector2 get_star_pos(int i, int j, Vector2 off_center)
{
 Vector2 results   = { 0 };
 float parallax[4] = { 0.80, 0.60, 0.30, 0.00 };

 results.x = stars[i][j].x - (off_center.x * parallax[i]);
 results.y = stars[i][j].y - (off_center.y * parallax[i]);

 return results;
}




void draw_stars()
{
 int   i         = 0;
 int   j         = 0;
 float angle     = 0.0;
 float alpha     = 0.0;
 float blend     = 0.0;
 float scale     = 0.0;
 float halfway   = 0.0;
 double s_offset = 0.0;

 float scaling[4]   = { 0.70, 0.80, 0.90, 1.00 };
 float frequency[4] = { 1.00, 0.80, 0.50, 0.30 };
 float opacity[4]   = { 0.38, 0.58, 0.74, 1.00 };

 Vector2 pos        = { 0 };
 Vector2 off_center = { 0 };
 Vector2 p_shift    = { 0 };

 Color colors[4]    = { WHITE, BLUE, CHILL_CYAN, SKY_BLUE };


 // Give the world a light blue background
 //
 tex_blit(tex_stars, 0, 0, SCREEN_W, SCREEN_H, tex_stars.width, tex_stars.height, 0.0, 9.0, Fade(WHITE, 0.75));
 DrawRectangle(0, 0, SCREEN_W * MAP_SIZE, SCREEN_H * MAP_SIZE, Fade(BLUE, 0.25));

 /// DEBUG: draw box at center of area
 /// DrawRectangle(SCREEN_W-40, SCREEN_H-40, 80, 80, Fade(ORANGE, 0.75));


 // Calc distance from center of map to current camera target
 // Scale this to determine how far to shift stars per-layer
 //
 off_center.x = ((SCREEN_W * MAP_SIZE) / 2.0) - camera.target.x;
 off_center.y = ((SCREEN_H * MAP_SIZE) / 2.0) - camera.target.y;

 for(i=0; i<4; i++) {
     alpha = opacity[i];

     // First pass draws normally
     //
     for(j=0; j<(int)(STAR_DENSITY * frequency[i]); j++) {
         p_shift = get_star_pos(i, j, off_center);
         scale = stars[i][j].z;
         scale *= scaling[i];

         if(is_odd(j)) { angle = star_vars[i][j].x + anims[10]; }
         else          { angle = star_vars[i][j].x - anims[10]; }

         tex_blit(tex_star, 0, 0, p_shift.x, p_shift.y, 64, 64, angle, scale, Fade(colors[(int)stars[i][j].w], alpha));
                                                         }

     // Second pass draws affected stars
     // Switching blend mode on/off repeatedly kills performance
     //
     BeginBlendMode(BLEND_ADDITIVE);
     for(j=0; j<(int)(STAR_DENSITY * frequency[i]); j++) {
         if(star_vars[i][j].y > 0.0) {
             p_shift = get_star_pos(i, j, off_center);
             scale = stars[i][j].z;
             scale *= scaling[i];

             if(is_odd(j)) { angle = star_vars[i][j].x + anims[10]; }
             else          { angle = star_vars[i][j].x - anims[10]; }

             scale *= rescale(star_vars[i][j].y, 0.0, 1.0, 1.0, 1.5);
             tex_blit(tex_star, 0, 0, p_shift.x, p_shift.y, 64, 64, angle, scale, Fade(colors[(int)stars[i][j].w], star_vars[i][j].y * 0.50));

             scale *= rescale(star_vars[i][j].y, 0.0, 1.0, 1.0, 1.7);
             tex_blit(tex_star, 0, 0, p_shift.x, p_shift.y, 64, 64, angle, scale, Fade(colors[(int)stars[i][j].w], star_vars[i][j].y * 0.50));
                                     }
                                                         }
     EndBlendMode();
                    }

 return;
}




void draw_node_connections()
{
 int i          = 0;
 int j          = 0;
 int on_path    = 0;
 float alpha    = 0.10;
 Vector2 _start = { 0 };
 Vector2 _end   = { 0 };
 Color col      = WHITE;

 for(i=0; i<MAX_NODES; i++) {
     if(nodes[i].enabled) {

         _start.x = nodes[i].pos.x;
         _start.y = nodes[i].pos.y;

         for(j=0; j<MAX_LINKS; j++) {
             if(nodes[i].links[j] != -1) {

                 // Examine path and see if we need to highlight this link
                 //
                 col   = WHITE;
                 alpha = 0.10;
                 if(flags.path_locked) {
                     on_path = examine_path(i, nodes[i].links[j]);
                     if(on_path) { col = YELLOW; alpha = 0.50; }
                                       }

                 _end.x = nodes[nodes[i].links[j]].pos.x;
                 _end.y = nodes[nodes[i].links[j]].pos.y;
                 DrawLineEx(_start, _end, 10.0, Fade(col, alpha));
                                         }
                                    }
                          }
                            }

 return;
}




void draw_nodes()
{
 int i = 0;
 int j = 0;
 float pulse    = 0.0;
 Vector2 dest   = { 0 };

 Color col = BLANK;

 for(i=0; i<MAX_NODES; i++) {
     if(nodes[i].enabled) {
         // Draw planet
         //
         if(nodes[i].planet_cel > 0) {
             dest.x = nodes[i].pos.x;
             dest.y = nodes[i].pos.y;
             tex_blit(tex_64x, (64 * nodes[i].planet_cel), 0, dest.x, dest.y, 64, 64, nodes[i].planet_anim, 2.0, WHITE);
                                     }

         // Draw node marker
         //
         if(flags.node_selected == i) {
             pulse = sin(TO_RADIANS(anims[4])) * 2.0;
             col = YELLOW;
                                      }
         else {
             pulse = 0.0;
             col = WHITE;
              }

         DrawRing(nodes[i].pos,          0, 10 + pulse, 0.0, 360.00,  16, Fade(col, 0.60));
         DrawRing(nodes[i].pos, 13 + pulse * 2, 16 + pulse * 2, 0.0, 360.00,  16, Fade(col, 0.60));

         // Draw ships in orbit
         //
         for(j=0; j<MAX_SHIPS; j++) {
             if(ships[j].enabled) {
                 if(ships[j].orbiting == i) { draw_ship(i, j, 0.0, 0.0); }
                                  }
                                    }

         draw_node_name(i);
                          }
                            }


 // Ships that are not orbiting nodes need to be drawn
 // (This all got janky in a hurry lol)
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].orbiting == -1) {
             ///tex_blit(tex_32x, (32 * ships[i].cel_x), 0, ships[i].pos.x, ships[i].pos.y, 32, 32, 0.0, 3.0, WHITE);
             draw_ship(-1, i, ships[i].pos.x, ships[i].pos.y);
                                     }
                         }
                            }

 // Draw node names
 //

 return;
}




// NOTE: If [NODE] is -1, then we are not locked to a node, and will
//       draw to the absolute coordinates instead
//
void draw_ship(int _node, int _ship, float abs_x, float abs_y)
{
 Vector2 offset = { 0 };
 Vector2 dest   = { 0 };
 float   width  = 4.0;
 float   angle  = 0;
 float   scale  = 1.0;

 if(_ship < 0  || _ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, _ship); return; }
 if(_node < -1 || _node >= MAX_NODES) { log(LOG_ALWAYS, "Node OOB in call to %s. Node = %d", __func__, _node); return; }

 // advance offsets (this should really go in the delta updates lol)
 //
 offset = advance(ships[_ship].orbit_anim, 80);

 if(_node > -1) {
     dest.x = nodes[_node].pos.x + offset.x;
     dest.y = nodes[_node].pos.y + offset.y;
                }
 else {
     dest.x = abs_x;
     dest.y = abs_y;
      }

 ships[_ship].last_pos.x = dest.x;
 ships[_ship].last_pos.y = dest.y;

 angle = ships[_ship].orbit_anim;

 switch(ships[_ship].team) {
     case 20: // Voidstar
              tex_blit(tex_voidstar, 0, 0, dest.x, dest.y, 128, 128, angle,       1.0, Fade(PINK, 0.35));
              tex_blit(tex_voidstar, 0, 0, dest.x, dest.y, 128, 128, angle + 120, 1.0, Fade(PINK, 0.35));
              tex_blit(tex_voidstar, 0, 0, dest.x, dest.y, 128, 128, angle + 240, 1.0, Fade(PINK, 0.35));
              break;

     case 21: // Warp Coil
              tex_blit(tex_64x, 960, 0, dest.x, dest.y, 64, 64, 0.0, 2.0, WHITE);
              break;

     case 24: // Celestial Fire
              if(_node > -1) {
                  dest.x = nodes[_node].pos.x;
                  dest.y = nodes[_node].pos.y;
                             }
              else {
                  dest.x = abs_x;
                  dest.y = abs_y;
                   }

              BeginBlendMode(BLEND_ADDITIVE);
              tex_blit(tex_celestial_b, 0, 0, dest.x, dest.y, 173, 173, angle,       scale, Fade(WHITE, 0.30));
              tex_blit(tex_celestial_b, 0, 0, dest.x, dest.y, 173, 173, angle + 60,  scale, Fade(WHITE, 0.30));
              tex_blit(tex_celestial_b, 0, 0, dest.x, dest.y, 173, 173, angle + 120, scale, Fade(WHITE, 0.30));

              tex_blit(tex_celestial_b, 0, 0, dest.x, dest.y, 173, 173, anims[15],     scale, Fade(WHITE, 0.20));
              tex_blit(tex_celestial_b, 0, 0, dest.x, dest.y, 173, 173, anims[15] * 2, scale, Fade(WHITE, 0.20));

              tex_blit(tex_celestial_r, 0, 0, dest.x, dest.y, 173, 173, angle + 180, scale, Fade(WHITE, 0.30));
              tex_blit(tex_celestial_r, 0, 0, dest.x, dest.y, 173, 173, angle + 240, scale, Fade(WHITE, 0.30));
              tex_blit(tex_celestial_r, 0, 0, dest.x, dest.y, 173, 173, angle + 300, scale, Fade(WHITE, 0.30));

              tex_blit(tex_celestial_r, 0, 0, dest.x, dest.y, 173, 173, 180 + anims[15],     scale, Fade(WHITE, 0.20));
              tex_blit(tex_celestial_r, 0, 0, dest.x, dest.y, 173, 173, 180 + anims[15] * 2, scale, Fade(WHITE, 0.20));
              EndBlendMode();
              break;

     default: // All other ships
              tex_blit(tex_32x, (32 * ships[_ship].cel_x), 0, dest.x, dest.y, 32, 32, 0.0, 3.0, WHITE);
              break;
                           }


 // Draw wanted level, if player
 //
 if(!_ship) {
     if(flags.wanted_anim > 0.001) {
         tex_blit(tex_l_stripe, anims[8], 0, dest.x, dest.y - 90, 128, 32, 0.0, 2.0, Fade(PINK, flags.wanted_anim * 0.75));

         justify_text_xy("WANTED", dest.x + 4, dest.y - 112, font1, Fade(BLACK, flags.wanted_anim), 3.0, 6.0);
         justify_text_xy("WANTED", dest.x,     dest.y - 116, font1, Fade(WHITE, flags.wanted_anim), 3.0, 6.0);

         ///tex_blit(tex_32x, (32 * 6), 0, dest.x, dest.y - 96, 32, 32, 0.0, 3.0, Fade(WHITE, flags.wanted_anim));
                                   }
            }

 return;
}




// NOTE: Draws warning on screen if all containers have been covered
//
void draw_warning()
{
 char blergh[80];

 tex_blit(tex_stripes, anims[8], 0, 0, 232, SCREEN_W, 32, 0.0, 2.0, Fade(ORANGE, 0.75));
 justify_text("W  A  R  N  I  N  G", 208, font1, WHITE, 3.0, 3.0);

 //sprintf(blergh, "%d", flags.covered_countdown);
 //justify_text(blergh, 130, font1, Fade(YELLOW, 0.50), 12.0, 12.0);


 return;
}




void fire_shot(int shooter, int gun, float x, float y, float z, int target, int direction)
{
 float off_z = 0;
 int who     = -1;

 who = spawn_radiance(x, y, z + off_z, target, direction ^ 1);

 if(who != -1) {
     actors[who].gun      = gun;
     actors[who].color    = guns[gun].col;
     actors[who].v_speed  = guns[gun].velocity;
     actors[who].cel_x    = guns[gun].cel_x;
     actors[who].atk_buff = actors[shooter].atk_buff; // Copy this value into shot for later
               }

 //SetSoundVolume(sfx[guns[gun].sfx], scale(flags.rapid_fire, 0, 12, 0.20, 0.10));
 PlaySound(sfx[guns[gun].sfx]);

 if(flags.rapid_fire < 12) { flags.rapid_fire++; }


 // Check for fire effects
 //
 fire_effect(shooter, gun);

 return;
}




void fire_effect(int shooter, int gun)
{
 if(shooter < 0 || shooter >= MAX_ACTORS) { log(LOG_ALWAYS, "shooter OOB in all to %s. Shooter = %d", __func__, shooter); return; }
 if(gun < 0 || gun >= MAX_GUNS)           { log(LOG_ALWAYS, "gun OOB in all to %s. gun = %d", __func__, gun); return;             }

 int target = -1;
 int rnjesus = 0;

 switch(gun) {
     case 0: // Vampimp
             break;

     case 1: // Psychocat
             break;

     case 2: // LuvBot
             break;

     case 3: /// Siren
             // Pick a friendly target. If damaged, heal damage.
             // if not damaged, apply offensive buff (27 is atk buff up)
             target = pick_target_to_heal(0);
             if(target != -1) {
                 spawn_aura(target, 26, 3.0, GREEN);
                 actors[target].stamina += 15;
                 if(actors[target].stamina > 100) { actors[target].stamina = 100; }
                              }
             break;

     case 4: // Grunt
             break;

     case 5: // Assault Bot
             break;

     case 6: /// Inquisitor
             // Pick an enemy target. Apply DEF or ATK debuff
             target = pick_target(0);
             if(target != -1) {
                 rnjesus = quick_rand(100);
                 if(is_odd(rnjesus)) {
                     log(LOG_1, "Inquisitor applied a DEF DEBUFF");
                     spawn_aura(target, 28, 3.0, RED); // Def down
                     actors[target].def_buff = -1;
                                     }
                 else {
                     log(LOG_1, "Inquisitor applied a ATK DEBUFF");
                     spawn_aura(target, 25, 3.0, RED); // Atk down
                     actors[target].atk_buff = -1;
                      }
                              }
             break;

     case 7: /// Shield Goon
             // Pick a friendly target. Apply DEF buff
             target = pick_target(1);
             if(target != -1) {
                 spawn_aura(target, 29, 3.0, LIME);
                 actors[target].def_buff = 1;
                              }
             break;
             }

 return;
}





// NOTE:
//
bool is_game_over()
{

 return(0);
}





void kill_player()
{

 flags.game_ending     = 1;

 flags.tint_col        = BLACK;
 flags.tint_str        = 0.0;
 flags.tint_velocity   = 0.007;

 flags.mosaic          = 0.0;
 flags.mosaic_velocity = 0.007;

 flags.game_speed      = 1.0;
 flags.game_velocity   = -0.0050;

 PlaySound(sfx[17]);
 PlaySound(sfx[83]);

 return;
}





void draw_node_name(int what)
{
 if(what < 0 || what >= MAX_NODES) { log(LOG_ALWAYS, "what OOB in call to %s. What = %d", __func__, what); return; }

 float name_scale   = 4.0;
 float name_spacing = 4.0;
 int scan_distance  = SCREEN_W;
 int dist           = 0;
 int k              = 0;
 int can_scan       = 0;

 Color col = WHITE;
 Color cargoset[4] = { CARGO_RED,   CARGO_YELLOW, CARGO_CYAN,  CARGO_VIOLET };


 if(flags.node_selected == what) { col = YELLOW; }
 else                            { col = WHITE;  }

 justify_text_xy(nodes[what].name, nodes[what].pos.x + 4, nodes[what].pos.y + 34, font2, BLACK, name_scale, name_spacing);
 justify_text_xy(nodes[what].name, nodes[what].pos.x, nodes[what].pos.y + 30, font2, col, name_scale, name_spacing);


 // If the player has a scanner, we can draw what's available on the planet
 // The normal scanner picks up nodes within one screen radius
 // Advanced scanner shows all planets in the system
 //
 if(flags.scanner_level == 1) {
     dist = dist_2d(ships[0].pos.x, ships[0].pos.y, nodes[what].pos.x, nodes[what].pos.y);
     if(dist < scan_distance) { can_scan = 1; }
                              }
 if(flags.scanner_level == 2) { can_scan = 1; }

 if(nodes[what].scanned) { can_scan = 1; }


 if(can_scan) {
     if(nodes[what].planet_cel > 0) {
         k = nodes[what].cargo_offer;
         tex_blit_c(tex_8x, (8 * 15), 0, nodes[what].pos.x-64-16, nodes[what].pos.y + 55, 8, 8, 4.0, WHITE);
         tex_blit_c(tex_8x, 64 + (8 * k), 0, nodes[what].pos.x-32-16, nodes[what].pos.y + 55, 8, 8, 4.0, cargoset[k]);

         k = nodes[what].cargo_pref;
         tex_blit_c(tex_8x, (8 * 14), 0, nodes[what].pos.x+32-16, nodes[what].pos.y + 55, 8, 8, 4.0, WHITE);
         tex_blit_c(tex_8x, 64 + (8 * k), 0, nodes[what].pos.x+64-16, nodes[what].pos.y + 55, 8, 8, 4.0, cargoset[k]);
                                    }
              }

 return;
}




void draw_notice()
{
 float alpha = 1.0;
 Color fuzzset[4]  = { LIME,        CARGO_YELLOW, CREW_ORANGE, CARGO_RED    };

 if(flags.notice_anim <= 0.10) { alpha = rescale(flags.notice_anim, 0.0,  0.10, 0.0, 1.0); }
 if(flags.notice_anim >= 0.90) { alpha = rescale(flags.notice_anim, 0.90, 1.00, 1.0, 0.0); }

 EndMode2D();
 BeginShaderMode(edge_fade);

 switch(flags.show_notice) {
     case 1: // Voidstar moving
             tex_blit(tex_stripes, anims[8], 0, 0, 300, SCREEN_W, 38, 0.0, 2.0, Fade(PINK, alpha * 0.75));
             justify_text("VOIDSTAR ON THE MOVE", 276, font1, Fade(WHITE, alpha), 3.0, 3.0);
             break;

     case 2: // Warp Coil
             tex_blit(tex_stripes, anims[8], 0, 0, 300, SCREEN_W, 38, 0.0, 2.0, Fade(LIME, alpha * 0.75));
             justify_text("WARP COIL ON THE MOVE", 276, font1, Fade(WHITE, alpha), 3.0, 3.0);
             break;

     case 3: // Passing Turn
             tex_blit(tex_stripes, anims[8], 0, 0, 300, SCREEN_W, 38, 0.0, 2.0, Fade(CREW_ORANGE, alpha * 0.75));
             justify_text("PASSING TIME", 276, font1, Fade(WHITE, alpha), 3.0, 3.0);
             break;

     case 4: // Fuzz despawn
             tex_blit(tex_stripes, anims[8], 0, 0, 300, SCREEN_W, 38, 0.0, 2.0, Fade(fuzzset[flags.fuzz_level], alpha * 0.75));
             justify_text("PATROL SHIP LEAVING SYSTEM", 276, font1, Fade(WHITE, alpha), 3.0, 3.0);
             break;
                           }

 EndShaderMode();
 BeginMode2D(camera);

 return;
}




void draw_hud_manifest()
{
 int   i       = 0;
 int   j       = 0;
 int   total   = ships[0].crew_count;
 float start_x = 0;

 char blergh[40];

 Color crewset[4]  = { CREW_ORANGE, CREW_BLUE,    CREW_GREEN,  CREW_PURPLE  };
 Color cargoset[4] = { CARGO_RED,   CARGO_YELLOW, CARGO_CYAN,  CARGO_VIOLET };
 Color fuzzset[4]  = { LIME,        CARGO_YELLOW, CREW_ORANGE, CARGO_RED    };
 Color col         = BLANK;

 EndMode2D();


 // Draw fuzz level
 //
 start_x = SCREEN_W - 100;

 BeginShaderMode(edge_fade);
     if(flags.sky_on_fire) { col = BLACK;                     }
     else                  { col = fuzzset[flags.fuzz_level]; }

     tex_blit(tex_stripes, anims[8], 0, 0, 10, SCREEN_W, 32, 0.0, 2.0, Fade(col, 0.75));
     tex_blit(tex_stripes, anims[8], 0, 0, SCREEN_H - 48, SCREEN_W, 36, 0.0, 2.0, Fade(BLACK, 0.75));
 EndShaderMode();

 if(flags.sky_on_fire) {
     justify_text_r("FUZZ OFFLINE", SCREEN_W - 10 + 4, 10 + 4, font2, BLACK, 4.0, 4.0);
     justify_text_r("FUZZ OFFLINE", SCREEN_W - 10,     10,     font2, WHITE, 4.0, 4.0);
                       }
 else {
     justify_text_r(fuzz_titles[flags.fuzz_level], SCREEN_W - 10 + 4, 10 + 4, font2, BLACK, 4.0, 4.0);
     justify_text_r(fuzz_titles[flags.fuzz_level], SCREEN_W - 10,     10,     font2, WHITE, 4.0, 4.0);
      }


 // Draw Retirement fund and coffer
 //
 sprintf(blergh, "SAVED: %05d", flags.score[0]);
 justify_text_l(blergh, 10 + 4, 10 + 4, font2, BLACK, 4.0, 4.0);
 justify_text_l(blergh, 10,     10,     font2, YELLOW, 4.0, 4.0);

 sprintf(blergh, "ON HAND: %05d", flags.score[1]);
 justify_text_l(blergh, 500 + 4, 10 + 4, font2, BLACK, 4.0, 4.0);
 justify_text_l(blergh, 500,     10,     font2, CHILL_CYAN, 4.0, 4.0);


 // Draw crew manifest
 //
 ///total += ships[0].crew[0];
 ///total += ships[0].crew[1];
 ///total += ships[0].crew[2];
 ///total += ships[0].crew[3];

 start_x = ((float)SCREEN_W / 2.0) - (20 * total);

 for(i=0; i<4; i++) {
     for(j=0; j<ships[0].crew[i]; j++) {
         tex_blit_c(tex_8x, (8 * i), 0, start_x, SCREEN_H - 84, 8, 8, 4.0, crewset[i]);
         start_x += 40;
                                       }
                    }


 // Draw cargo manifest
 //
 total = 0;
 total += ships[0].cargo[0];
 total += ships[0].cargo[1];
 total += ships[0].cargo[2];
 total += ships[0].cargo[3];

 start_x = ((float)SCREEN_W / 2.0) - (20 * total);

 for(i=0; i<4; i++) {
     for(j=0; j<ships[0].cargo[i]; j++) {
         tex_blit_c(tex_8x, 64 + (8 * i), 0, start_x, SCREEN_H - 44, 8, 8, 4.0, cargoset[i]);
         start_x += 40;
                                        }
                    }


 BeginMode2D(camera);

 return;
}





// NOTE: Starts a cycle where all ships advance to their new positions
//       terminates after a set period of time. Then you have another
//       routine that resolves events at new positions.
//
void next_round(bool pass_turn)
{
 int i = 0;
 int j = 0;
 int _node         = 0;
 int linkage       = 0;
 int done          = 0;
 int rnjesus       = 0;
 int ship_node     = 0;

 int total_links   = 0;
 int total_fuzz    = 0;
 int total_salvage = 0;
 int max_fuzz      = 0;
 int max_salvage   = 2;
 int heal_factor   = 0;
 int remap[4]      = { 1, 3, 0, 2 };


 // If passing the turn, set up a notice
 //
 if(pass_turn) { flags.show_notice = 3; flags.notice_anim = 0.0; }

 // Node updates
 //
 if(!flags.tutorial) {
     for(i=0; i<MAX_NODES; i++) {
         if(nodes[i].enabled) {
             // Scan and celestial fire decay
             //
             if(nodes[i].scanned > 0) { nodes[i].scanned--; }

             if(nodes[i].fire_count > 0) { nodes[i].fire_count--; }

             // Inventory replenishment
             //
             if(nodes[i].planet_cel > -1) {
                 nodes[i].planet_reload--;
                 if(nodes[i].planet_reload <= 0) {
                     if(nodes[i].cargo_count < MAX_HOES) { nodes[i].cargo_count++; }
                     nodes[i].planet_reload = 6;
                                                 }

             // Inventory cycling
             //
             nodes[i].planet_mutate--;
             if(nodes[i].planet_mutate <= 0) {
                 nodes[i].planet_mutate = 30 + quick_rand(20);
                 nodes[i].cargo_offer = quick_rand(4);
                 nodes[i].cargo_pref  = quick_rand(4);

                 if(nodes[i].cargo_offer == nodes[i].cargo_pref) {
                     nodes[i].cargo_offer = remap[nodes[i].cargo_pref];
                                                                 }
                                             }
                                          }
                              }
                                }
                     }

 // Each ship should choose a new destination and de-orbit
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         switch(ships[i].team) {
             case 0: // Player
                     if(flags.node_selected < 0) { ships[i].skip_turn = 1; break; }
                     if(ships[i].going_to < 0)   { ships[i].skip_turn = 1; break; }
                     ships[i].origin.x = ships[i].last_pos.x;
                     ships[i].origin.y = ships[i].last_pos.y;

                     ships[i].orbiting = -1;
                     PlaySound(sfx[100]);
                     break;

             case 1: // Enemy (has a flat 15% chance of fucking off per-round)
                     // Enemy may also choose to skip turn if player is en route to their node
                     ships[i].stamina--;

                     if(flags.tutorial) { ships[i].skip_turn = 1; }
                     else {
                         if(ships[i].bribed) { ships[i].bribed--; }
                         if(quick_rand(100) < 15) { ships[i].skip_turn = 1; }
                         else                     { ship_wander(i, 0); }
                          }
                     break;
                               }
                          }
                            }


 // Move this elsewhere?
 //
 flags.round_anim = 0.0;

 do {
     if(pass_turn) { flags.notice_anim = flags.round_anim; }

     advance_ships();
     update_and_render();
     if(flags.round_anim >= 1.00) { done = 1; }

    } while(!done);


 if(pass_turn) { flags.show_notice = 0; flags.notice_anim = 0.0; }


 // Ships should orbit their destinations
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled && ships[i].team < 2 && !ships[i].skip_turn) {
         ships[i].orbiting = ships[i].going_to;
         ships[i].skip_turn = 0;
         ships[i].going_to = -1;
                                                                      }

     if(ships[i].skip_turn) { ships[i].skip_turn = 0; }
                            }


 // Inventory ships on the board
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].team == 22) { total_salvage++; }
         if(ships[i].team == 23) { total_salvage++; }
         if(ships[i].team == 24) { total_salvage++; }
         if(ships[i].team == 1)  { total_fuzz++; }
                          }
                            }


 /// ------------------------------------------------------------------
 ///                  Tutorial stops updates here
 /// ------------------------------------------------------------------
 if(flags.tutorial) { return; }


 // Check for Fuzz escalation
 //
 flags.total_rounds++;
 if(flags.total_rounds < 100)  { flags.fuzz_level = 0; max_fuzz = 3; }
 if(flags.total_rounds >= 100) { flags.fuzz_level = 1; max_fuzz = 4; }
 if(flags.total_rounds >= 200) { flags.fuzz_level = 2; max_fuzz = 5; }
 if(flags.total_rounds >= 300) { flags.fuzz_level = 3; max_fuzz = 6; }


 // Check for Fuzz spawn-despawn
 //
 flags.fuzz_respawn--;
 if(flags.fuzz_respawn <= 0 && flags.fuzz_pending == -1) {
     if(total_fuzz < max_fuzz) { flags.fuzz_pending = 1; }
     flags.fuzz_respawn = 15 + quick_rand(15);
                                                         }


 if(flags.wanted_level > 0) { flags.wanted_level--; }


 // Check for Misc Events (salvage, rescue, celestial fire) re-spawn
 //
 flags.salvage_respawn--;
 if(flags.salvage_respawn <= 0) {
     if(total_salvage < max_salvage) { create_misc_event(quick_rand(3), -1); }
     flags.salvage_respawn = 50 + quick_rand(25);
                                }


 // Check for healing based off of location
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].team < 2) { // Player or Fuzz
             heal_factor = 3;
             _node = ships[i].orbiting;

             if(_node != -1) {
                 if(nodes[_node].planet_cel) { heal_factor = 7; } // orbiting a planet
                 heal_crew(i, heal_factor);
                 log(LOG_1, "Healing ship %d for %d", i, heal_factor);
                             }
                                }
                          }
                            }


 // Check for voidstar and warp coil movement
 //
 rnjesus = quick_rand(100);
 if(rnjesus < 2) { warp_ship_type_event(20, 1); }

 rnjesus = quick_rand(100);
 if(rnjesus < 2) { warp_ship_type_event(21, 2); }


 // Check for ship despawn
 //
 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].team == 1) { // fuzz only
             if(ships[i].stamina <= 0) { remove_ship_event(i, 4); }
                                }
                          }
                            }

 return;
}




// NOTE: Moves ship positions to their destinations
//       based off the value of flags.round_anim (0.0 - 1.0)
//
void advance_ships()
{
 int i           = 0;
 int ship_node   = 0;
 Vector2 offset  = { 0 };
 Vector2 new_pos = { 0 };
 Vector2 end_pos = { 0 };


 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled && ships[i].team < 2 && !ships[i].skip_turn) {
         ship_node = ships[i].going_to;

         offset = advance(ships[i].orbit_anim, 80);

         end_pos.x = nodes[ship_node].pos.x + offset.x;
         end_pos.y = nodes[ship_node].pos.y + offset.y;

         new_pos.x = tan_scale(flags.round_anim, 0.0, 1.0, ships[i].origin.x, end_pos.x);
         new_pos.y = tan_scale(flags.round_anim, 0.0, 1.0, ships[i].origin.y, end_pos.y);

         ships[i].pos.x = new_pos.x;
         ships[i].pos.y = new_pos.y;
                                                                      }
                            }

 return;
}




// NOTE: This is called for a given ship, so we can take some things
//       for granted...
//
void advance_a_ship(int ship)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }

 int ship_node   = 0;
 Vector2 offset  = { 0 };
 Vector2 end_pos = { 0 };
 Vector2 new_pos = { 0 };

 ship_node = ships[ship].going_to;

 offset = advance(ships[ship].orbit_anim, 80);

 end_pos.x = nodes[ship_node].pos.x + offset.x;
 end_pos.y = nodes[ship_node].pos.y + offset.y;

 new_pos.x = tan_scale(flags.round_anim, 0.0, 1.0, ships[ship].origin.x, end_pos.x);
 new_pos.y = tan_scale(flags.round_anim, 0.0, 1.0, ships[ship].origin.y, end_pos.y);

 ships[ship].pos.x = new_pos.x;
 ships[ship].pos.y = new_pos.y;

 return;
}




// NOTE: Lets a ship pick a random path based off of the current node links
//       Contents of the destination node are not taken into consideration
//
// NOTE: If the ship is fuzz (team == 1) there is a slight chance that
//       they will attempt pathfinding to get closer to the player.
//       If that fails, they will just rando-wander like always
//
void ship_wander(int ship, bool forced)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }

 int j           = 0;
 int result      = 0;
 int hunting     = 0;
 int target_node = 0;
 int ship_node   = 0;
 int total_links = 0;
 int rnjesus     = 0;
 int wait_chance = 50;

 ship_node   = ships[ship].orbiting;
 total_links = 0;

 // If not forced into moving, a non-player ship
 // has a 50% chance of waiting for the player to run into them
 // This will end their turn
 //
 if(ship > 0 && !forced) {
     if(ships[0].going_to == ships[ship].orbiting) {
         rnjesus = quick_rand(100);
         if(rnjesus < wait_chance) { ships[ship].skip_turn = 1; return; }
                                                   }
                         }

 if(ships[ship].team == 1) {
     rnjesus = quick_rand(100);
     if(rnjesus < (10 + (flags.fuzz_level * 4))) { hunting = 1; log(LOG_1, "ship %d is hunting", ship); }
                           }

 if(hunting) {
     // Look at what player is going before path-finding. Is it
     // in orbit, or going somewhere??
     //
     if(ships[0].orbiting != -1) { target_node = ships[0].orbiting; }
     else                        { target_node = ships[0].going_to; }

     log(LOG_ALWAYS, "Hunting. Ship %d going to node %d", ship, target_node);

     result = find_path(ship, ships[ship].orbiting, target_node);
     if(result && node_path[1] >= 0) {
         ships[ship].going_to = node_path[1];
         ships[ship].origin.x = ships[ship].last_pos.x;
         ships[ship].origin.y = ships[ship].last_pos.y;

         ships[ship].orbiting = -1;
         return;
                                     }
             }

 // Normal wandering
 //
 for(j=0; j<MAX_LINKS; j++) {
     if(nodes[ship_node].links[j] != -1) { total_links++; }
                            }

 rnjesus = quick_rand(total_links);
 ships[ship].going_to = nodes[ship_node].links[rnjesus];
 ships[ship].origin.x = ships[ship].last_pos.x;
 ships[ship].origin.y = ships[ship].last_pos.y;

 ships[ship].orbiting = -1;

 return;
}





// NOTE: This is terribly lazy and does not scale well.
//       I can get away with this because we're jamming this shit out.
//
int find_node(Vector2 mouse_pos)
{
 int i    = 0;
 int dist = 0;

 for(i=0; i<MAX_NODES; i++) {
     dist = dist_2d((int)mouse_pos.x, (int)mouse_pos.y, (int)nodes[i].pos.x, (int)nodes[i].pos.y);
     if(dist < 40) { return(i); }
                            }

 return(-1);
}




void clear_path()
{
 int i = 0;

 for(i=0; i<MAX_NODES; i++) { node_path[i] = -1; draw_path[i] = -1; }

 return;
}




void clear_nodes()
{
 int i = 0;

 for(i=0; i<MAX_NODES; i++) {
     nodes[i].pos.x       = 0;
     nodes[i].pos.y       = 0;
     nodes[i].accessory   = 0;
     nodes[i].anim_state  = 0;
     nodes[i].anim_var    = 0;
     nodes[i].links[0]    = -1;
     nodes[i].links[1]    = -1;
     nodes[i].links[2]    = -1;
     nodes[i].links[3]    = -1;
     nodes[i].links[4]    = -1;
     nodes[i].links[5]    = -1;
     nodes[i].planet_reload = 0;
     nodes[i].planet_mutate = 0;
     nodes[i].planet_cel  = 0;
     nodes[i].planet_anim = 0;
     nodes[i].fire_radius = 0.0;
     nodes[i].fire_count  = 0;
     nodes[i].mapped      = 0;
     nodes[i].enabled     = 0;
     nodes[i].named       = 0;
     nodes[i].role        = 0;

     nodes[i].cargo_offer = 0;
     nodes[i].cargo_count = 0;
     nodes[i].cargo_pref  = 0;
     nodes[i].scanned     = 0;
     nodes[i].band_hue    = 0;

     sprintf(nodes[i].name, "DEADBEEF");
                            }

 return;
}




// NOTE: This is the start of a recursive node traversal
//       system. It uses the below routine for recursion,
//       and the one below that for recording the solution.
//
// NOTE: Returns 1 if a path has been found, else 0
//
int find_path(int ship, int start_node, int dest_node)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(0); }
 if(dest_node < 0 || dest_node >= MAX_NODES) { log(LOG_ALWAYS, "dest node OOB in call to %s. dest = %d", __func__, dest_node); return(0); }
 int i = 0;

 // Init arrays and values
 //
 for(i=0; i<MAX_NODES; i++) {
     node_traversed[i] = 0;
     node_path[i]  = -1;
     node_queue[i] = -1;
                            }

 queue_depth = 0;
 queue_distance = 999;


 // Mark starting node as traversed
 //
 node_traversed[start_node] = 1;


 // Push start node to queue
 //
 node_queue[queue_depth] = start_node;
 queue_depth++;


 // Start recursion
 //
 for(i=0; i<MAX_LINKS; i++) {
     if(nodes[start_node].links[i] != -1) { traversal(ship, nodes[start_node].links[i], dest_node); }
                            }


 // If the node path hasn't been filled out,
 // we never found a valid solution.
 //
 if(node_path[0] == -1) { return(0); }
 else                   { return(1); }
}




// NOTE: Recursive sheot
//
void traversal(int ship, int current_node, int dest_node)
{
 int i = 0;
 int __node = 0;
 bool done  = 0;

 // Mark starting node as traversed
 //
 node_traversed[current_node] = 1;


 // Push node to queue
 //
 node_queue[queue_depth] = current_node;
 queue_depth++;


 // Check if current node is destination
 //
 if(current_node == dest_node) {
     if(queue_depth < queue_distance) {
         record_path(ship, queue_depth - 1);
         queue_distance = queue_depth;
                                      }
     done = 0;
                               }


 // Continue recursing if we're not at our destination
 //
 if(!done) {
     for(i=0; i<MAX_LINKS; i++) {
         __node = nodes[current_node].links[i];

         if(__node != -1) {
             if(!node_traversed[__node]) { traversal(ship, __node, dest_node); }
                          }
                                }
           }


 // Pop queue and clear traversal
 //
 if(queue_depth > 0) { queue_depth--; }
 node_traversed[current_node] = 0;

 return;
}




// NOTE: Records current path using the queue at its
//       current depth. Will erase any previous solution.
//
void record_path(int ship, int depth)
{
 if(depth < 0 || depth >= MAX_NODES) { log(LOG_ALWAYS, "Depth OOB in call to %s. Depth = %d", __func__, depth); return; }
 int i = 0;

 for(i=0; i<MAX_NODES; i++) { node_path[i] = -1; }
 if(!ship) { for(i=0; i<MAX_NODES; i++) { draw_path[i] = -1; } }

 log(LOG_1, "RECORDING PATH. Depth = %d", depth);

 // Player ship gets its own copy, for drawing to screen
 //
 for(i=depth; i>=0; i--) {
     node_path[i] = node_queue[i];
     if(!ship) { draw_path[i] = node_queue[i]; }

     log(LOG_1, "%d", node_path[i]);
                         }

 return;
}




// NOTE: Examines a path solution and returns 1 if
//       the supplied node and link are a part of the solution.
//       Otherwise, returns 0
//
int examine_path(int node, int linked_node)
{
 int i = 0;

 if(draw_path[0] == node && draw_path[1] == linked_node)                     { return(1); }
 if(draw_path[MAX_NODES-1] == node && draw_path[MAX_NODES-2] == linked_node) { return(1); }

 for(i=1; i<MAX_NODES-1; i++) {
     if(draw_path[i] == node &&
       (draw_path[i-1] == linked_node || draw_path[i+1] == linked_node)) { return(1); }

                              }

 if(draw_path[0] == linked_node && draw_path[1] == node)                     { return(1); }
 if(draw_path[MAX_NODES-1] == linked_node && draw_path[MAX_NODES-2] == node) { return(1); }

 for(i=1; i<MAX_NODES-1; i++) {
     if(draw_path[i] == linked_node &&
       (draw_path[i-1] == node || draw_path[i+1] == node)) { return(1); }

                              }

 return(0);
}






void parse_map(Image image)
{
 int w           = image.width;
 int h           = image.height;
 int x           = 0;
 int y           = 0;
 int i           = 0;
 int j           = 0;
 int k           = 0;
 int r           = 0;
 int _linkage    = 0;
 int _total      = 0;
 int rnjesus     = 0;
 int name_offset = 0;
 Color node_col  = BLANK;
 Color link_col  = BLANK;
 Color _NODE     = (Color){ 255,  72,   0, 255 };
 Color _RING     = (Color){ 108,  94, 181, 255 };

 int remap[4]   = { 1, 3, 0, 2 };
 int off_x[16]  = { -2, -1,  0,  1,  2,     2,  2,  2,     2,  1,  0, -1, -2,    -2, -2, -2 };
 int off_y[16]  = { -2, -2, -2, -2, -2,    -1,  0,  1,     2,  2,  2,  2,  2,     1,  0, -1 };

 // If this gets too large we'll have to declare elsewhere
 //
 Color map_data[MAX_NODES][MAX_LINKS];


 log(LOG_1, "Map scanning: w = %d, h = %d", w, h);
 name_offset = quick_rand(79);

 // We should assume that this will already have happened,
 // but it's a cheap operation and I don't trust myself
 //
 clear_nodes();

 for(i=0; i<MAX_NODES; i++) {
     for(j=0; j<MAX_LINKS; j++) { map_data[i][j] = BLACK; } // NB: Black, *NOT* blank
                            }


 // Scan node links and create basic variables
 //
 for(y=0; y<h; y++) {
     for(x=0; x<w; x++) {
         node_col = GetImageColor(image, x, y);

         if(compare_color(node_col, _NODE)) { // Node marker
             rnjesus = quick_rand(100);
             nodes[_total].pos.x      = 27.0 * (float)x;
             nodes[_total].pos.y      = 21.5 * (float)y;
             nodes[_total].accessory  = 0;
             nodes[_total].anim_state = 0;
             nodes[_total].anim_var   = 0;
             nodes[_total].mapped     = 1;
             nodes[_total].enabled    = 1;
             nodes[_total].named      = 1;
             sprintf(nodes[_total].name, "%s", planet_names[name_offset]);

             name_offset++;
             if(name_offset > 79) { name_offset = 0; }

             log(LOG_1, "Found node at %d, %d", x, y);


             // Scan link ring
             //
             _linkage = 0;
             for(i=0; i<16; i++) {
                 link_col = GetImageColor(image, x + off_x[i], y + off_y[i]);
                 if(!compare_color(link_col, _RING)) {
                     map_data[_total][_linkage] = link_col;
                     _linkage++;

                     if(_linkage >= MAX_LINKS) { log(LOG_ALWAYS, "Too many links at map %d, %d", x, y); break; }
                                                     }
                                 }

             _total++;
                                            }
                        }
                    }


 // Second phase
 // Scan nodes for link matches (brute force because range is tiny)
 //
 for(i=0; i<MAX_NODES; i++) {
     for(j=0; j<MAX_LINKS; j++) {
         if(!compare_color(map_data[i][j], BLACK)) {

             for(k=0; k<MAX_NODES; k++) {
                 if(k != i) {
                     for(r=0; r<MAX_LINKS; r++) {
                         if(compare_color(map_data[i][j], map_data[k][r])) { nodes[i].links[j] = k; }
                                                }

                            }
                                        }
                                                   }
                                }
                            }


 // Create 18 planets
 // this should be a function of node count, but... there's only one real map
 //
 for(i=0; i<18; i++) {
     k = get_empty_node(0);
     if(k != -1) {
         nodes[k].role          = 4;
         nodes[k].band_hue      = quick_rand_f(360.0);
         nodes[k].planet_cel    = 1 + quick_rand(9);
         nodes[k].planet_anim   = quick_rand(360);
         nodes[k].cargo_offer   = quick_rand(4);
         nodes[k].cargo_count   = 2 + quick_rand(4);
         nodes[k].cargo_pref    = quick_rand(4);
         nodes[k].planet_mutate = 30 + quick_rand(20);
         nodes[k].planet_reload = 6;
         if(nodes[k].cargo_offer == nodes[k].cargo_pref) {
             nodes[k].cargo_offer = remap[nodes[k].cargo_pref];
                                                         }
                 }
                     }

 return;
}





int bacon_menu(int enemy_ship)
{
 int event      = -1;
 int esc_chance = 0;
 int can_escape = 0;
 int slot       = 1;
 int min_bribe  = 0;
 int rnjesus    = 0;
 flags.in_menu  = slot;

 if(enemy_ship < 0 || enemy_ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Enemy ship is OOB in %s. Ship = %d", __func__, enemy_ship); return(0); }


 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = ships[enemy_ship].level + 1;
 menus[slot].title       = ships[enemy_ship].level + 10;
 menus[slot].extras      = 100 + enemy_ship;
 open_menu(slot);
 PlaySound(sfx[85]);


 // If we don't have enough cash, we can't bribe
 //
 min_bribe = 2000 * (flags.fuzz_level + 1);
 if((flags.coffer < min_bribe) || flags.wanted_level || flags.tutorial) { menus[slot].invalid[1] = 1; }
 else                                                                   { menus[slot].invalid[1] = 0; }


 // If the ship is too fast, and we don't have upgraded engines...
 //
 if(!flags.engine_level) {
     if(ships[enemy_ship].level > 1) { esc_chance = 0;  }
     if(ships[enemy_ship].level < 2) { esc_chance = 25; }
                         }
 else {
     if(ships[enemy_ship].level > 1) { esc_chance = 35; }
     if(ships[enemy_ship].level < 2) { esc_chance = 75; }
      }

 rnjesus = quick_rand(100);
 if((rnjesus < esc_chance && !flags.tutorial) && !flags.wanted_level) { menus[slot].invalid[2] = 0; log(LOG_1, "ESCAPE IS POSSIBLE - %d of %d", rnjesus, esc_chance);   }
 else                                                                 { menus[slot].invalid[2] = 1; log(LOG_1, "PLAYER CANNOT ESCAPE - %d of %d", rnjesus, esc_chance); }



 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
          case 0: log(LOG_1, "User FOUGHT!");
                  flags.combat_pending = enemy_ship;
                  PlaySound(sfx[52]);
                  break;

          case 1: if(menus[slot].invalid[1]) { error_bleep(); event = -1;                     }
                  else                       { flags.coffer -= min_bribe; PlaySound(sfx[52]); }
                  log(LOG_1, "User BRIBED LIKE A PRO!");
                  break;

          case 2: if(menus[slot].invalid[2]) { error_bleep(); event = -1; }
                  else                       { PlaySound(sfx[52]);        }
                  log(LOG_1, "User GTFO'd!");
                  break;
                   }

    } while(event == -1);


 close_menu(slot);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 return(event);
}




void pimpstation_menu()
{
 int event        = -1;
 int slot         = 2;
 int exit         = 0;
 int rnjesus      = 0;
 int entry_coffer = flags.coffer;

 flags.in_menu = slot;

 /// MOVE THIS INTO ITS OWN ROUTINE TO REDUCE JANK
 // Update availability versus our coffers, and crew capacity
 //
 count_crew(0);

 if(flags.coffer < crew_cost[0] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[0] = 1; }
 else                                                                       { menus[slot].invalid[0] = 0; }

 if(flags.coffer < crew_cost[1] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[1] = 1; }
 else                                                                       { menus[slot].invalid[1] = 0; }

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 30 + quick_rand(2);
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User asked for VAMPIMP!");
                 if(menus[slot].invalid[0]) { error_bleep(); }
                 else {
                     assign_crew(0, 0);
                     flags.coffer -= crew_cost[0];
                     flags.total_spent += crew_cost[0];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 1: log(LOG_1, "User asked for PSYCHOCAT!");
                 if(menus[slot].invalid[1]) { error_bleep(); }
                 else {
                     assign_crew(0, 1);
                     flags.coffer -= crew_cost[1];
                     flags.total_spent += crew_cost[1];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 2: log(LOG_1, "User said fuck this shit and LEFT ORBIT!");
                 exit = 1;
                 break;
                   }

     // Update availability versus our coffers, and crew capacity
     //
     count_crew(0);

     if(flags.coffer < crew_cost[0] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[0] = 1; }
     else                                                                       { menus[slot].invalid[0] = 0; }

     if(flags.coffer < crew_cost[1] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[1] = 1; }
     else                                                                       { menus[slot].invalid[1] = 0; }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 // Evaluate spending
 //
 if(entry_coffer >= 5000) {
     if(((float)flags.coffer / (float)entry_coffer) <= 0.10) {
         rnjesus = quick_rand(100);
         if(rnjesus < 99) { PlaySound(sfx[228]); }
                                                             }
                          }

 return;
}




void advanced_pimping_menu()
{
 int event        = -1;
 int slot         = 3;
 int exit         = 0;
 int rnjesus      = 0;
 int entry_coffer = flags.coffer;

 flags.in_menu = slot;

 count_crew(0);

 if(flags.coffer < crew_cost[2] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[0] = 1; }
 else                                                                       { menus[slot].invalid[0] = 0; }

 if(flags.coffer < crew_cost[3] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[1] = 1; }
 else                                                                       { menus[slot].invalid[1] = 0; }

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 32 + quick_rand(2);
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User asked for LUVBOT!");
                 if(menus[slot].invalid[0]) { error_bleep(); }
                 else {
                     assign_crew(0, 2);
                     flags.coffer -= crew_cost[2];
                     flags.total_spent += crew_cost[2];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 1: log(LOG_1, "User asked for SIREN!");
                 if(menus[slot].invalid[1]) { error_bleep(); }
                 else {
                     assign_crew(0, 3);
                     flags.coffer -= crew_cost[3];
                     flags.total_spent += crew_cost[3];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 2: log(LOG_1, "User said fuck this shit and LEFT ORBIT!");
                 exit = 1;
                 break;
                   }

     // Update availability versus our coffers, and crew capacity
     //
     count_crew(0);

     if(flags.coffer < crew_cost[2] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[0] = 1; }
     else                                                                       { menus[slot].invalid[0] = 0; }

     if(flags.coffer < crew_cost[3] || ships[0].crew_count == flags.crew_level) { menus[slot].invalid[1] = 1; }
     else                                                                       { menus[slot].invalid[1] = 0; }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 // Evaluate spending
 //
 if(entry_coffer >= 5000) {
     if(((float)flags.coffer / (float)entry_coffer) <= 0.10) {
         rnjesus = quick_rand(100);
         if(rnjesus < 99) { PlaySound(sfx[228]); }
                                                             }
                          }

 return;
}





void starship_pimping_menu()
{
 int event        = -1;
 int slot         = 4;
 int exit         = 0;
 int rnjesus      = 0;
 int entry_coffer = flags.coffer;

 flags.in_menu = slot;


 /// MOVE THIS INTO ITS OWN ROUTINE TO REDUCE JANK
 // Update availability versus our coffers
 //
 if(flags.coffer < upgrade_cost[0] || flags.crew_level == MAX_CREW_LEVEL) { menus[slot].invalid[0] = 1; }
 else                                                                     { menus[slot].invalid[0] = 0; }

 if(flags.coffer < upgrade_cost[1] || flags.cargo_level == MAX_CARGO_LEVEL) { menus[slot].invalid[1] = 1; }
 else                                                                       { menus[slot].invalid[1] = 0; }

 if(flags.coffer < upgrade_cost[2] || flags.scanner_level != 1) { menus[slot].invalid[2] = 1; }
 else                                                           { menus[slot].invalid[2] = 0; }

 if(flags.coffer < upgrade_cost[3] || flags.engine_level == MAX_ENGINE_LEVEL) { menus[slot].invalid[3] = 1; }
 else                                                                         { menus[slot].invalid[3] = 0; }

 // Activate menu and hold until it's open
 //
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User asked for CREW UPGRADE!");
                 if(menus[slot].invalid[0]) { error_bleep(); }
                 else {
                     flags.crew_level++;
                     flags.coffer -= upgrade_cost[0];
                     flags.total_spent += upgrade_cost[0];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 1: log(LOG_1, "User asked for CARGO UPGRADE!");
                 if(menus[slot].invalid[1]) { error_bleep(); }
                 else {
                     flags.cargo_level++;
                     flags.coffer -= upgrade_cost[1];
                     flags.total_spent += upgrade_cost[1];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 2: log(LOG_1, "User asked for SCANNER UPGRADE!");
                 if(menus[slot].invalid[2]) { error_bleep(); }
                 else {
                     flags.scanner_level++;
                     flags.coffer -= upgrade_cost[2];
                     flags.total_spent += upgrade_cost[2];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 3: log(LOG_1, "User asked for ENGINE UPGRADE!");
                 if(menus[slot].invalid[3]) { error_bleep(); }
                 else {
                     flags.engine_level++;
                     flags.coffer -= upgrade_cost[3];
                     flags.total_spent += upgrade_cost[3];
                     PlaySound(sfx[52]);
                      }
                 break;

         case 4: log(LOG_1, "User said fuck this shit and LEFT ORBIT!");
                 exit = 1;
                 break;
                   }

     // Update availability versus our coffers
     //
     if(flags.coffer < upgrade_cost[0] || flags.crew_level == MAX_CREW_LEVEL) { menus[slot].invalid[0] = 1; }
     else                                                                     { menus[slot].invalid[0] = 0; }

     if(flags.coffer < upgrade_cost[1] || flags.cargo_level == MAX_CARGO_LEVEL) { menus[slot].invalid[1] = 1; }
     else                                                                       { menus[slot].invalid[1] = 0; }

     if(flags.coffer < upgrade_cost[2] || flags.scanner_level != 1) { menus[slot].invalid[2] = 1; }
     else                                                           { menus[slot].invalid[2] = 0; }

     if(flags.coffer < upgrade_cost[3] || flags.engine_level == MAX_ENGINE_LEVEL) { menus[slot].invalid[3] = 1; }
     else                                                                         { menus[slot].invalid[3] = 0; }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 // Evaluate spending
 //
 if(entry_coffer >= 5000) {
     if(((float)flags.coffer / (float)entry_coffer) <= 0.10) {
         rnjesus = quick_rand(100);
         if(rnjesus < 99) { PlaySound(sfx[228]); }
                                                             }
                          }

 return;
}





void cargo_menu()
{
 int i          = 0;
 int event      = -1;
 int slot       = 5;
 int exit       = 0;
 int to_xfer    = 0;
 int bonus_rate = 3;
 float hue      = 0;

 flags.in_menu  = slot;


 // SPECIAL CASE: Band color is set at-random during map parsing
 //
 hue = nodes[ships[0].orbiting].band_hue;
 menus[slot].band_color = ColorFromHSV(quick_rand_f(hue), 0.25, 0.5);


 // Update availability versus our coffers, and crew capacity
 //
 flags.cargo_total  = ships[0].cargo[0] + ships[0].cargo[1] + ships[0].cargo[2] + ships[0].cargo[3];
 flags.cargo_wanted = 0;
 for(i=0; i<4; i++) {
     if(i != nodes[ships[0].orbiting].cargo_offer) { flags.cargo_wanted += ships[0].cargo[i]; }
                    }

 if(flags.cargo_total == flags.cargo_level ||
    !nodes[ships[0].orbiting].cargo_count)  { menus[slot].invalid[0] = 1; }
 else                                       { menus[slot].invalid[0] = 0; }

 if(flags.cargo_wanted < 1) { menus[slot].invalid[1] = 1; }
 else                       { menus[slot].invalid[1] = 0; }

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 10 + (ships[0].orbiting % 5);
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User asked for PICKUP!");
                 if(menus[slot].invalid[0]) { error_bleep(); }
                 else {
                     to_xfer = (flags.cargo_level - flags.cargo_total);
                     log(LOG_1, "Pickup: %d slots available in ship", to_xfer);
                     log(LOG_1, "Pickup: %d units available on planet", nodes[ships[0].orbiting].cargo_count);
                     if(nodes[ships[0].orbiting].cargo_count < to_xfer) {
                         to_xfer = nodes[ships[0].orbiting].cargo_count;
                         log(LOG_1, "Pickup: Less on planet than we can carry. Adjusted units is %d", to_xfer);
                                                                        }
                     ships[0].cargo[nodes[ships[0].orbiting].cargo_offer] += to_xfer;
                     nodes[ships[0].orbiting].cargo_count -= to_xfer;
                     log(LOG_1, "Pickup: final unit count on planet %d", nodes[ships[0].orbiting].cargo_count);
                     PlaySound(sfx[52]);
                      }
                 break;

         // Unload all cargo that's not what's on offer
         // Preferential cargo gets a bonus rate
         case 1: log(LOG_1, "User asked for DROPOFF!");
                 if(menus[slot].invalid[1]) { error_bleep(); }
                 else {
                     for(i=0; i<4; i++) {
                         if(i != nodes[ships[0].orbiting].cargo_offer) {
                             to_xfer = ships[0].cargo[i];
                             flags.total_deliveries += to_xfer;
                             log(LOG_1, "Dropoff: %d units of type %d transferred", to_xfer, i);
                             if(i == nodes[ships[0].orbiting].cargo_pref) { add_funds(to_xfer * cargo_cost[i] * bonus_rate); }
                             else                                         { add_funds(to_xfer * cargo_cost[i]);              }
                             ships[0].cargo[i] = 0;
                                                                       }
                                        }
                     PlaySound(sfx[51]);
                      }
                 break;

         case 2: log(LOG_1, "User said fuck this shit and LEFT ORBIT!");
                 exit = 1;
                 break;
                   }

     // Update availability versus our coffers, and crew capacity
     //
     flags.cargo_total = ships[0].cargo[0] + ships[0].cargo[1] + ships[0].cargo[2] + ships[0].cargo[3];
     flags.cargo_wanted = 0;
     for(i=0; i<4; i++) {
         if(i != nodes[ships[0].orbiting].cargo_offer) { flags.cargo_wanted += ships[0].cargo[i]; }
                        }

     if(flags.cargo_total == flags.cargo_level ||
        !nodes[ships[0].orbiting].cargo_count) { menus[slot].invalid[0] = 1; }
     else                                      { menus[slot].invalid[0] = 0; }

     if(flags.cargo_wanted < 1) { menus[slot].invalid[1] = 1; }
     else                       { menus[slot].invalid[1] = 0; }

    } while(!exit);


 // SPECIAL: Visited planets will show their assets for a while
 //
 nodes[ships[0].orbiting].scanned = 10;


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 return;
}





// NOTE: This is the special ship summary menu that opens when
//       the player pounds the button in the bottom right of the screen
//
void summary_menu()
{
 int event = -1;
 int slot  = 9;
 int exit  = 0;
 flags.in_menu = slot;

 // Activate menu and hold until it's open
 //
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
     if(!IsMouseButtonDown(MOUSE_BUTTON_LEFT)) { flags.m_hold[0] = 0; }
    } while(menus[slot].anim < 1.00 || flags.m_hold[0]);


 do {
     event = check_button_events();
     update_and_render();
     if(event == 1) { exit = 1; }
    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 return;
}






void voidstar_menu()
{
 int i          = 0;
 int event      = -1;
 int slot       = 20;
 int exit       = 0;
 int rnjesus    = 0;

 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 20;
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User acknowledged their grim fate");
                 exit = 1;
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);


 // Calculate how fubar the player is
 //
 scan_roster(1);

 for(i=0; i<MAX_CREW; i++) {
     if(flags.friends[i] != -1) {
         rnjesus = quick_rand(100);
         if(is_odd(rnjesus)) {
             flags.total_deaths++;
             load_coffin(0, actors[flags.friends[i]].type);
             kill_actor(flags.friends[i]);
                             }
                                }
                           }

 return;
}





void warp_coil_menu()
{
 int i          = 0;
 int event      = -1;
 int slot       = 21;
 int exit       = 0;

 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 21;
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User acknowledged their amusing fate");
                 exit = 1;
                 PlaySound(sfx[101]);
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 warp_ship(0);

 return;
}





void salvage_menu(int _node)
{
 int i          = 0;
 int derelict   = -1;
 int event      = -1;
 int slot       = 22;
 int exit       = 0;
 int rnjesus    = 0;

 if(_node < 0 || _node >= MAX_NODES) { log(LOG_ALWAYS, "Node OOB in call to %s. Node = %d", __func__, _node); return; }

 // Randomly determine contents of the derelict
 //
 rnjesus = quick_rand(5);

 switch(rnjesus) {
     case 0: // Credits
             menus[slot].extras = 22;
             break;

     case 1: // Cargo hold Upgrade
             if(flags.cargo_level == MAX_CARGO_LEVEL) { menus[slot].extras = 22; }
             else                                     { menus[slot].extras = 23; }
             break;

     case 2: // Basic Scanner
             if(flags.scanner_level == 0) { menus[slot].extras = 24; }
             else                         { menus[slot].extras = 22; }
             break;

     case 3: // Crew Upgrade
             if(flags.crew_level == MAX_CREW_LEVEL) { menus[slot].extras = 22; }
             else                                   { menus[slot].extras = 25; }
             break;

     case 4: // Health packs and credits
             menus[slot].extras = 26;
             break;
                 }


 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 22 + quick_rand(4);
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User found some savory salvage");
                 exit = 1;
                 PlaySound(sfx[86]);
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].orbiting == _node && ships[i].team == 22) { clone_ship(i, 0); }
                          }
                            }

 // We have to clear the role for this node,
 // Because it doesn't belong to the ship, but the node (Weird, I know)
 //
 nodes[_node].role = 0;


 // Evaluate salvage results for player
 //
 switch(menus[slot].extras) {
    case 22: // Credits
             add_funds(5000 + quick_rand(1000));
             break;

    case 23: // Cargo hold Upgrade
             flags.cargo_level++;
             break;

    case 24: // Basic Scanner
             flags.scanner_level = 1;
             break;

    case 25: // Crew hold upgrade
             flags.crew_level++;
             break;

    case 26: // Health packs and credits
             heal_crew(0, 100);
             add_funds(3000 + quick_rand(1000));
             break;
                 }

 return;
}





void rescue_menu(int _node)
{
 int i          = 0;
 int event      = -1;
 int slot       = 27;
 int exit       = 0;
 int total      = 0;
 int rnjesus    = 0;

 if(_node < 0 || _node >= MAX_NODES) { log(LOG_ALWAYS, "Node OOB in call to %s. Node = %d", __func__, _node); return; }

 // Randomly determine contents of the ship being rescued
 // (Cargo type)
 //
 rnjesus = quick_rand(4);

 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 34;
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User rescued a fellow pimp");
                 exit = 1;
                 PlaySound(sfx[86]);
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].orbiting == _node && ships[i].team == 23) { clone_ship(i, 0); }
                          }
                            }

 // We have to clear the role for this node,
 // Because it doesn't belong to the ship, but the node (Weird, I know)
 //
 nodes[_node].role = 0;


 // Evaluate salvage results for player
 //
 add_funds(1000 + quick_rand(1000));

 flags.cargo_total  = ships[0].cargo[0] + ships[0].cargo[1] + ships[0].cargo[2] + ships[0].cargo[3];
 total = (flags.cargo_level - flags.cargo_total);
 if(total > 0) { ships[0].cargo[rnjesus] += total; }

 return;
}




void celestial_fire_menu(int _node)
{
 int i          = 0;
 int event      = -1;
 int slot       = 28;
 int exit       = 0;
 int total      = 0;
 int rnjesus    = 0;

 if(_node < 0 || _node >= MAX_NODES) { log(LOG_ALWAYS, "Node OOB in call to %s. Node = %d", __func__, _node); return; }

 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 35;
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User activated a celestial fire artifact");
                 exit = 1;
                 PlaySound(sfx[86]);
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].orbiting == _node && ships[i].team == 24) { clone_celestial_fire(i); }
                          }
                            }

 // We have to clear the role for this node,
 // Because it doesn't belong to the ship, but the node (Weird, I know)
 //
 nodes[_node].role = 0;
 nodes[_node].fire_count = 30;
 nodes[_node].fire_radius = 0.0;

 return;
}






void create_player_ship(int slot, int force_node)
{
 if(slot < 0 || slot >= MAX_SHIPS) { log(LOG_ALWAYS, "Slot OOB in call to %s. Slot = %d", __func__, slot); return; }

 int i = slot;
 int k = 0;

 clear_ship(i);

 ships[i].enabled  = 1;
 ships[i].named    = 1;
 ships[i].origin.x = 0.0;
 ships[i].origin.y = 0.0;
 ships[i].team     = 0;
 ships[i].cel_x    = 5;
 ships[i].going_to = -1;
 ships[i].orbiting = 0;
 ships[i].orbit_anim = quick_rand(360);

 if(!flags.tutorial) {
     assign_crew(i, 0);
     assign_crew(i, 0);
     assign_crew(i, 1);
                     }

 ships[i].cargo[0] = 0;
 ships[i].cargo[1] = 0;
 ships[i].cargo[2] = 0;
 ships[i].cargo[3] = 0;
 sprintf(ships[i].name, "THE LURVE BOAT");


 // Place player ship in random space,
 // and set camera target to match
 //
 if(force_node != -1) { k = force_node;        }
 else                 { k = get_empty_node(0); }

 ships[i].pos.x = nodes[k].pos.x;
 ships[i].pos.y = nodes[k].pos.y;
 ships[i].orbiting = k;

 camera.target.x = ships[i].pos.x;
 camera.target.y = ships[i].pos.y;

 return;
}




int create_fuzz_ship(int enemy_level, int force_node)
{
 int rnjesus = 0;
 int i       = get_free_ship();
 int _node   = get_empty_node(1);
 if(i == -1 || _node == -1) { return(-1); }

 if(force_node != -1) { _node = force_node; }

 log(LOG_1, "fuzz created at slot %d", i);
 clear_ship(i);

 ships[i].enabled  = 1;
 ships[i].stamina  = 75 + quick_rand(25);
 ships[i].level    = enemy_level;
 ships[i].named    = 1;
 ships[i].pos.x    = nodes[_node].pos.x;
 ships[i].pos.y    = nodes[_node].pos.y;
 ships[i].origin.x = 0.0;
 ships[i].origin.y = 0.0;
 ships[i].team     = 1;      // Hostile Team
 ships[i].cel_x    = 16 + enemy_level;
 ships[i].going_to = -1;
 ships[i].orbiting = _node;
 ships[i].orbit_anim = quick_rand(360);

 rnjesus = quick_rand(50);
 sprintf(ships[i].name, "%s", fuzz_names[rnjesus]);

 if(flags.tutorial) { return(i); }

 switch(enemy_level) {
     case 0: // 2-3 enemies
             assign_crew(i, 4);
             assign_crew(i, 4);

             rnjesus = quick_rand(100);
             if(is_odd(rnjesus)) { assign_crew(i, 4); };
             break;

     case 1: // 3-4 enemies
             rnjesus = quick_rand(3);
             switch(rnjesus) {
                 case 0:
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         break;

                 case 1:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         break;

                 case 2:
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         break;
                             }
             break;

     case 2: // 4-5 enemies
             rnjesus = quick_rand(3);
             switch(rnjesus) {
                 case 0:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         break;

                 case 1:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         break;

                 case 2:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         assign_crew(i, 6);
                         break;
                             }
             break;

     case 3: // 5-6 enemies
             rnjesus = quick_rand(3);
             switch(rnjesus) {
                 case 0:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         assign_crew(i, 7);
                         break;

                 case 1:
                         assign_crew(i, 4);
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         assign_crew(i, 7);
                         assign_crew(i, 7);
                         break;

                 case 2:
                         assign_crew(i, 4);
                         assign_crew(i, 5);
                         assign_crew(i, 5);
                         assign_crew(i, 6);
                         assign_crew(i, 7);
                         assign_crew(i, 7);
                         break;
                             }
             break;
                     }


 return(i);
}



void create_pimpstations(int quantity, int force_node)
{
 int i        = 0;
 int who      = 0;
 int stations = quantity;
 int _node    = 0;

 for(i=0; i<stations; i++) {
     who = get_free_ship();

     if(force_node < 0) { _node = get_empty_node(0); }
     else               { _node = force_node;       }

     log(LOG_1, "pimp station #%d, at node %d", i, _node);

     if(who != -1 && _node != -1) {
         clear_ship(who);

         ships[who].enabled  = 1;
         ships[who].named    = 1;
         ships[who].pos.x    = nodes[_node].pos.x;
         ships[who].pos.y    = nodes[_node].pos.y;
         ships[who].origin.x = 0.0;
         ships[who].origin.y = 0.0;
         ships[who].team     = 2;   // Friendly neutral team
         ships[who].cel_x    = 7;
         ships[who].going_to = -1;
         ships[who].orbiting = _node;
         ships[who].orbit_anim = quick_rand(360);
         sprintf(ships[who].name, "PIMPSTATION");

         nodes[_node].role = 1;     // Mark this node as occupied by the station
                                  }
                           }

 return;
}




void create_advanced_stations(int quantity, int force_node)
{
 int i        = 0;
 int who      = 0;
 int stations = quantity;
 int _node    = 0;

 for(i=0; i<stations; i++) {
     who = get_free_ship();

     if(force_node < 0) { _node = get_empty_node(0); }
     else               { _node = force_node;       }

     log(LOG_1, "advanced station #%d, at node %d", i, _node);

     if(who != -1 && _node != -1) {
         clear_ship(who);

         ships[who].enabled  = 1;
         ships[who].named    = 1;
         ships[who].pos.x    = nodes[_node].pos.x;
         ships[who].pos.y    = nodes[_node].pos.y;
         ships[who].origin.x = 0.0;
         ships[who].origin.y = 0.0;
         ships[who].team     = 2;   // Friendly neutral team
         ships[who].cel_x    = 8;
         ships[who].going_to = -1;
         ships[who].orbiting = _node;
         ships[who].orbit_anim = quick_rand(360);
         sprintf(ships[who].name, "ADVANCED PIMPING");

         nodes[_node].role = 2;     // Mark this node as occupied by the station
                                  }
                           }

 return;
}




void create_upgrade_station(int quantity, int force_node)
{
 int i        = 0;
 int who      = 0;
 int stations = quantity;
 int _node    = 0;

 who   = get_free_ship();

 if(force_node < 0) { _node = get_empty_node(0); }
 else               { _node = force_node;       }

 log(LOG_1, "pimp my starship station #%d, at node %d", i, _node);

 if(who != -1 && _node != -1) {
     clear_ship(who);

     ships[who].enabled  = 1;
     ships[who].named    = 1;
     ships[who].pos.x    = nodes[_node].pos.x;
     ships[who].pos.y    = nodes[_node].pos.y;
     ships[who].origin.x = 0.0;
     ships[who].origin.y = 0.0;
     ships[who].team     = 2;   // Friendly neutral team
     ships[who].cel_x    = 9;
     ships[who].going_to = -1;
     ships[who].orbiting = _node;
     ships[who].orbit_anim = quick_rand(360);
     sprintf(ships[who].name, "PIMP MY STARSHIP");

     nodes[_node].role = 3;     // Mark this node as occupied by the station
                              }

 return;
}






void create_voidstars()
{
 int i        = 0;
 int stars    = 2;
 int who      = -1;
 int _node    = -1;

 for(i=0; i<stars; i++) {
     who   = get_free_ship();
     _node = get_empty_node(0);

     log(LOG_1, "voidstar #%d, at node %d", i, _node);

     if(who != -1 && _node != -1) {
         clear_ship(who);

         ships[who].enabled  = 1;
         ships[who].named    = 1;
         ships[who].pos.x    = nodes[_node].pos.x;
         ships[who].pos.y    = nodes[_node].pos.y;
         ships[who].origin.x = 0.0;
         ships[who].origin.y = 0.0;
         ships[who].team     = 20;   // Voidstar
         ships[who].going_to = -1;
         ships[who].orbiting = _node;
         ships[who].orbit_anim = quick_rand(360);

         sprintf(ships[who].name, "VOIDSTAR");

         nodes[_node].role = 20;     // Mark this node as having a voidstar
                                  }
                        }

 return;
}





void create_warp_coil()
{
 int i        = 0;
 int stars    = 1;
 int who      = -1;
 int _node    = -1;

 for(i=0; i<stars; i++) {
     who   = get_free_ship();
     _node = get_empty_node(0);

     log(LOG_1, "warp coil #%d, at node %d", i, _node);

     if(who != -1 && _node != -1) {
         clear_ship(who);

         ships[who].enabled  = 1;
         ships[who].named    = 1;
         ships[who].pos.x    = nodes[_node].pos.x;
         ships[who].pos.y    = nodes[_node].pos.y;
         ships[who].origin.x = 0.0;
         ships[who].origin.y = 0.0;
         ships[who].team     = 21;   // Warpcoil
         ships[who].going_to = -1;
         ships[who].orbiting = _node;
         ships[who].orbit_anim = quick_rand(360);

         sprintf(ships[who].name, "WARP COIL");

         nodes[_node].role = 21;     // Mark this node as having a warp coil
                                  }
                        }

 return;
}




// NOTE: [TYPE] is:
//       0 = Salvage
//       1 = Rescue
//       2 = Celestial Fire
//
void create_misc_event(int type, int force_node)
{
 int i         = 0;
 int quantity  = 1;
 int who       = -1;
 int _node     = -1;

 if(type < 0 || type >= 3) { log(LOG_ALWAYS, "Type OOB in call to %s. Type = %d", __func__, type); return; }

 for(i=0; i<quantity; i++) {
     who   = get_free_ship();

     if(force_node < 0) { _node = get_empty_node(0); }
     else               { _node = force_node;       }

     log(LOG_1, "map event #%d, at node %d", i, _node);

     if(who != -1 && _node != -1) {
         clear_ship(who);
         ships[who].enabled  = 1;
         ships[who].named    = 1;
         ships[who].pos.x    = nodes[_node].pos.x;
         ships[who].pos.y    = nodes[_node].pos.y;
         ships[who].origin.x = 0.0;
         ships[who].origin.y = 0.0;
         ships[who].going_to = -1;
         ships[who].orbiting = _node;
         ships[who].orbit_anim = quick_rand(360);

         switch(type) {
             case 0: // Salvage
                     nodes[_node].role   = 22;   // Node marker
                     ships[who].team     = 22;   // Derelict
                     ships[who].cel_x    = 15;
                     sprintf(ships[who].name, "DERELICT");
                     break;

             case 1: // Rescue
                     nodes[_node].role   = 23;   // Node marker
                     ships[who].team     = 23;   // Rescue
                     ships[who].cel_x    = 14;
                     sprintf(ships[who].name, "RESCUE");
                     break;

             case 2: // Celestial Fire
                     nodes[_node].role   = 24;   // Node marker
                     ships[who].team     = 24;   // Alien Artifact
                     ships[who].cel_x    = 14;
                     sprintf(ships[who].name, "CELESTIAL FIRE");
                     break;
                      }

                                  }
                           }

 return;
}






int get_free_ship()
{
 int i = 0;

 for(i=0; i<MAX_SHIPS; i++) {
     if(!ships[i].enabled) { return(i); }
                            }

 return(-1);
}





void clear_ship(int slot)
{
 if(slot < 0 || slot >= MAX_SHIPS) { log(LOG_ALWAYS, "Slot OOB in call to %s. Slot = %d", __func__, slot); return; }
 int i = 0;

 ships[slot].enabled    = 0;
 ships[slot].destruct   = 0;
 ships[slot].level      = 0;
 ships[slot].bribed     = 0;
 ships[slot].skip_turn  = 0;
 ships[slot].named      = 0;
 ships[slot].pos.x      = 0;
 ships[slot].pos.y      = 0;
 ships[slot].last_pos.x = 0;
 ships[slot].last_pos.y = 0;
 ships[slot].origin.x   = 0.0;
 ships[slot].origin.y   = 0.0;
 ships[slot].team       = 10;
 ships[slot].cel_x      = 0;
 ships[slot].going_to   = -1;
 ships[slot].orbiting   = -1;
 ships[slot].orbit_anim = 0;
 ships[slot].crew_count = 0;
 ships[slot].c_anim     = 0;
 ships[slot].stamina    = 0;

 for(i=0; i<MAX_CREW; i++)  {
     ships[slot].crew[i]    = 0;
     ships[slot].coffins[i] = -1;
                            }

 // Search actor roster and kill of actors that are ship's crew
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled) {
         if(actors[i].owner == slot) { kill_actor(i); }
                           }
                             }

 for(i=0; i<MAX_CARGO; i++) { ships[slot].cargo[i] = 0; }

 sprintf(ships[slot].name, "UNUSED");

 return;
}




// NOTE: If [MUST_BE_VACANT] is true, nothing can be in the
//       returned node. No planets, no ships. Nothing.
//
int get_empty_node(bool must_be_vacant)
{
 int i       = 0;
 int exit    = 0;
 int babort  = 500; // I mean, if we can't find something in 500 tries, something is truly fucked
 int vacant  = 0;
 int rnjesus = 0;

 do {
     rnjesus = quick_rand(MAX_NODES);

     if(must_be_vacant) {
         vacant = 1;

         for(i=0; i<MAX_SHIPS; i++) {
             if(ships[i].enabled) {
                 if(ships[i].orbiting == rnjesus) { vacant = 0; }
                                  }
                                    }
                        }
     else { vacant = 1; }

     if(nodes[rnjesus].mapped &&
        !nodes[rnjesus].planet_cel &&
        !nodes[rnjesus].role &&
        ships[0].orbiting != rnjesus &&
        vacant) { return(rnjesus); }

     babort--;
     if(babort < 0) { exit = 1; }
    } while(!exit);

 return(-1);
}




int find_event(int _node)
{
 int i = 0;

 if(nodes[_node].role > 0) { return(nodes[_node].role); }

 return(0); // Used to return -1, but this jammed up my shit logic
}




// NOTE: Assigns an actor of [type] to a [ship]
//
int assign_crew(int ship, int type)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(-1); }
 if(type < 0 || type >= MAX_CREW)  { log(LOG_ALWAYS, "type OOB in call to %s. type = %d", __func__, type); return(-1); }

 Color crewset[8] = { CREW_ORANGE, CREW_BLUE, CREW_GREEN, CREW_PURPLE, CARGO_RED, CARGO_YELLOW, SKILL_PINK, LIGHT_GRAY };

 int ce_x[8] = {  0,  5,  0,  5,   10, 10, 15, 15 };
 int ce_y[8] = {  0,  1,  1,  0,    0,  1,  0,  1 };

 int who = spawn_actor(0, 0, 0, 0, 16, 0.0, 0.0, 0.0, 0);
 if(who == -1) { log(LOG_ALWAYS, "Couldn't acquire actor assigning crew to ship"); return(-1); }


 if(!ship) { actors[who].flip = 1; }
 else      { actors[who].flip = 0; }

 actors[who].owner   = ship;
 actors[who].stamina = 100;
 actors[who].team    = ships[ship].team;
 actors[who].type    = type;
 actors[who].gun     = type;
 actors[who].visible = 0;
 actors[who].enabled = 1;

 actors[who].scale   = 4.0;
 actors[who].state   = 0;
 actors[who].effect  = 9;
 actors[who].cel_x   = ce_x[type];
 actors[who].cel_y   = ce_y[type];
 actors[who]._size_w = 32;
 actors[who]._size_h = 32;
 actors[who].color   = crewset[type];

 return(who);
}




// NOTE: Takes inventory of a ship's crew by looking over actor list
//       This will populate the ship's manifest, and also return a total crew count
//
int count_crew(int ship)
{
 int i          = 0;
 int crew_count = 0;

 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(-1); }

 // Clear existing crew count
 //
 for(i=0; i<MAX_CREW; i++) { ships[ship].crew[i] = 0; }


 // Scan actors and assign to ship's manifest
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled) {
         if(actors[i].owner == ship) {
             if(actors[i].type >= 0 && actors[i].type < MAX_CREW) {
                 ships[ship].crew[actors[i].type]++;
                 crew_count++;
                                                                  }
                                     }
                           }
                             }

 ships[ship].crew_count = crew_count;

 return(crew_count);
}





// NOTE: Lazy math.
//
void add_funds(int amount)
{
 flags.coffer     += (amount / 2);
 flags.retirement += (amount / 2);


 return;
}





void long_haul()
{
 int i     = 0;
 int event = 0;

 // If we arrived here by accident, bail
 //
 if(flags.node_selected == -1) { return; }


 // Move current selection into a perm variable
 //
 flags.long_haul = flags.node_selected;
 flags.follow_ship = 0;

 do {
     next_round(0);
     event = check_events();
     if(event) { clear_path_selection(); }
     else {
         flags.node_selected = flags.long_haul;

         flags.path_locked = find_path(0, ships[0].orbiting, flags.node_selected);
         flags.last_node   = flags.node_selected;

         if(flags.path_locked) { ships[0].going_to = node_path[1]; }
         else                  { ships[0].going_to = -1;           }
          }

         // We need at least one pass to update the ship's last position
         // or we can do it manually (it'll happen when drawing the frame)
         // But this loop can also slow down the jump between nodes (for stylistic reasons)
         //
         for(i=0; i<1; i++) { update_and_render(); }

    } while(ships[0].orbiting != flags.long_haul && !event);

 flags.long_haul     = -1;
 flags.node_selected = -1;
 flags.last_node     = -1;

 return;
}




// NOTE: Clears out path selection variables after a round has been
//       completed, or the user clicks off a node
//
void clear_path_selection()
{
 flags.path_locked    = 0;
 flags.node_selected  = -1;
 flags.last_node      = -1;
 flags.long_haul      = -1;
 flags.show_long_haul = 0;

 return;
}




// NOTE: This should be called from the autopilot menu, and maybe in general
//       to keep the player in-view. This performs error correction over time
//
void laggy_cam(int target_ship)
{
 Vector2 error    = { 0 };             // error calculation per axis
 Vector2 sweet    = { 0.50, 0.50 };    // Sweet spot is 1/2 of the screen.
 Vector2 deadband = { 0.05, 0.05 };    // Error must exceed deadband before action occurs
 Vector2 adjust   = { 0 };             // Adjustment rate for this frame

 if(target_ship < 0 || target_ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, target_ship); return; }

 // Calculate error rate for x and y axes
 //
 error.x = (ships[target_ship].pos.x - camera.target.x) / (float)SCREEN_W;
 error.y = (ships[target_ship].pos.y - camera.target.y) / (float)SCREEN_H;

 if(fabs(error.x) > deadband.x) { adjust.x = scale(fabs(error.x), 0.0, 1.0, 0.0, 20.0); }
 if(fabs(error.y) > deadband.y) { adjust.y = scale(fabs(error.y), 0.0, 1.0, 0.0, 20.0); }

 if(error.x > 0) { camera.target.x += adjust.x; }
 if(error.x < 0) { camera.target.x -= adjust.x; }

 if(error.y > 0) { camera.target.y += adjust.y; }
 if(error.y < 0) { camera.target.y -= adjust.y; }

 clip_camera();

 return;
}




void clip_camera()
{
 flags.min_cam_w   = (SCREEN_W * 0.5) / camera.zoom;
 flags.min_cam_h   = (SCREEN_H * 0.5) / camera.zoom;
 flags.max_cam_w   = (SCREEN_W * MAP_SIZE) - ((SCREEN_W * 0.5) / camera.zoom);
 flags.max_cam_h   = (SCREEN_H * MAP_SIZE) - ((SCREEN_H * 0.5) / camera.zoom);


 if(camera.target.x < flags.min_cam_w) { camera.target.x = flags.min_cam_w; }
 if(camera.target.x > flags.max_cam_w) { camera.target.x = flags.max_cam_w; }

 if(camera.target.y < flags.min_cam_h) { camera.target.y = flags.min_cam_h; }
 if(camera.target.y > flags.max_cam_h) { camera.target.y = flags.max_cam_h; }

 return;
}




void zoom_tracking()
{

 float error           = 0.0;             // error calculation
 float deadband        = 0.05;            // Deadband to exceed before action
 float adjust          = 0.0;

 float adjust_rate_in  = 0.016;           // adjustment rate (slow = 0.0075,  med = 0.01,  faster = 0.015)
 float adjust_rate_out = 0.023;           //

 if(flags.zoom_target > camera.zoom) {
     camera.zoom += adjust_rate_in;
     if(camera.zoom > flags.zoom_target) { camera.zoom = flags.zoom_target; }
                                     }

 if(flags.zoom_target < camera.zoom) {
     camera.zoom -= adjust_rate_out;
     if(camera.zoom < flags.zoom_target) { camera.zoom = flags.zoom_target; }
                                     }


 return;
}




// NOTE: This forces a single ship to advance to a different node
//       totally at random. Node contents are not taken into consideration
//
void force_retreat(int ship)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }

 int done = 0;

 ship_wander(ship, 1);


 // Move this elsewhere?
 //
 flags.round_anim = 0.0;

 do {
     advance_a_ship(ship);
     update_and_render();
     if(flags.round_anim >= 1.00) { done = 1; }

    } while(!done);


 // Ships should orbit their destinations
 //
 ships[ship].orbiting = ships[ship].going_to;
 ships[ship].skip_turn = 0;
 ships[ship].going_to = -1;


 return;
}






// NOTE: Place project-specific calls to drawing routines in here.
//       This is more or less a hook into the 4x framework
//
void draw_project_specific_items()
{
 if(flags.in_combat) { draw_combat(); }


 return;
}




// NOTE: Draws the combat scene, background, actors, etc
//
void draw_combat()
{
 Vector2 offset = { 0 };
 char blergh[80];
 int i = 0;
 int j = 0;

 float alpha   = flags.combat_anim;
 float spacing = scale(flags.combat_anim, 0.0, 1.0, 150.0, 3.0);
 float rnjesus = (float)quick_rand_f(100.0) / 100.0;

 EndMode2D();

 // Draw warning strips
 //
 BeginShaderMode(edge_fade);
     DrawRectangle(0, SCREEN_H - 420, SCREEN_W, 260, Fade(BLUE, alpha * 0.75));
     tex_blit(tex_stripes, anims[8], 0, 0, SCREEN_H - 320, SCREEN_W, 160, 0.0, 2.0, Fade(CARGO_RED, 0.75 * alpha * 0.75));
 EndShaderMode();


 // Draw title band
 //
 sprintf(blergh, "%s VERSUS THE LURVE BOAT", ships[flags.combat_pending].name);
 justify_text(blergh, SCREEN_H - 474, font1, Fade(WHITE, alpha), 3.0, 3.0);



 // Move actors into position
 //
 if(flags.combat_anim != 1.0) {
     for(i=0; i<MAX_CREW; i++) {
         j = flags.friends[i];
         if(j != -1) {
             actors[j].visible = 1;
             actors[j].cel   = tex_crew;
             actors[j].pos.x = scale(flags.combat_anim, 1.0, 0.0, (SCREEN_W - 500) + (75 * i), SCREEN_W + (300 * i));
             actors[j].pos.y = SCREEN_H - 200;
                     }

         j = flags.enemies[i];
         if(j != -1) {
             actors[j].visible = 1;
             actors[j].cel   = tex_crew;
             actors[j].pos.x = scale(flags.combat_anim, 1.0, 0.0, 500 - (75 * i), -(300 * i));
             actors[j].pos.y = SCREEN_H - 200;
                     }
                               }
                              }

 if(flags.combat_result == 0) {
     sprintf(blergh, "Y O U   W I N");
     for(j=0; j<8; j++) {
         offset = advance(anims[11] + 44 * j, 10);
         justify_text_xy(blergh, (SCREEN_W / 2) + offset.x, SCREEN_H - 400 + offset.y, font1, Fade(BLUE, alpha * 0.35), 3.0, spacing);
                        }
     justify_text(blergh, SCREEN_H - 400, font1, Fade(YELLOW, alpha), 3.0, spacing);
                              }

 if(flags.combat_result == 1) {
     sprintf(blergh, "Y O U   L O S E");
     for(j=0; j<8; j++) {
         offset = advance(anims[11] + 44 * j, 10);
         justify_text_xy(blergh, (SCREEN_W / 2) + offset.x, SCREEN_H - 400 + offset.y, font1, Fade(BLACK, alpha * 0.35), 3.0, spacing);
                        }
     justify_text(blergh, SCREEN_H - 400, font1, Fade(WHITE, alpha), 3.0, spacing);
                              }

 return;
}





void scan_roster(bool friends_only)
{
 int i          = 0;
 int j          = 0;
 int list_count = 0;


 for(i=0; i<MAX_CREW; i++) { flags.friends[i] = -1; flags.enemies[i] = -1; }


 // construct list of friendly participants (this is very lazy)
 //
 for(j=0; j<4; j++) {
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled &&
            !actors[i].team &&
            actors[i].type == j &&
            actors[i].owner == 0) { flags.friends[list_count] = i; list_count++; }
                                 }
                    }


 if(friends_only) { return; }


 // construct list of enemy participants (this is also very lazy)
 //
 list_count = 0;
 for(j=4; j<8; j++) {
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled &&
            actors[i].team == 1 &&
            actors[i].type == j &&
            actors[i].owner == flags.combat_pending) { flags.enemies[list_count] = i; list_count++; }
                                 }
                    }

 return;
}





// NOTE: Sets up combatants for combat and populates helper arrays
//
void prepare_for_combat(int enemy_ship)
{
 int i          = 0;
 int j          = 0;
 int list_count = 0;

 scan_roster(0);

 // construct list of friendly participants (this is very lazy)
 //
 for(j=0; j<4; j++) {
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled &&
            !actors[i].team &&
            actors[i].type == j &&
            actors[i].owner == 0) {
                actors[i].atk_buff = 0;
                actors[i].def_buff = 0;
                actors[i].blend    = 1.0;
                actors[i].extra[0] = 0;
                actors[i].extra[1] = quick_rand(guns[actors[i].gun].delay);
                actors[i].extra[2] = guns[actors[i].gun].ammo;
                                  }
                                 }
                    }


 // construct list of enemy participants (this is also very lazy)
 //
 for(j=4; j<8; j++) {
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled &&
            actors[i].team == 1 &&
            actors[i].type == j &&
            actors[i].owner == flags.combat_pending) {
                actors[i].atk_buff = 0;
                actors[i].def_buff = 0;
                actors[i].blend    = 1.0;
                actors[i].extra[0] = 0;
                actors[i].extra[1] = quick_rand(guns[actors[i].gun].delay);
                actors[i].extra[2] = guns[actors[i].gun].ammo;
                                                     }
                                 }
                    }

 // Trigger inbound combat animation
 //
 flags.combat_anim = 0.01;
 flags.in_combat = 1;


 // Crossfade music
 //
 fade_music_out(0);
 fade_music_in(1);

 return;
}




// NOTE: Fires off animations and sets variables for combatants
//       This routine should be called from the delta updates routine
//
void process_combat()
{
 int i         = 0;
 int j         = 0;
 int crew      = 0;
 int target    = 0;
 int f_size    = 0;
 int e_size    = 0;
 int delay     = 0;
 int ammo_left = 0;
 int alternate = 0;

 bool end_pass = 0;

 // NOTE: Code uses extra[] variables
 //       [0] = current state
 //       [1] = reload time left (in ticks)
 //       [2] = Ammo remaining
 //

 // Check for everyone being dead
 //
 for(i=0; i<MAX_CREW; i++) {
     if(flags.friends[i] != -1) {
         if(actors[flags.friends[i]].stamina > 0) { f_size++;    }
         if(actors[flags.friends[i]].extra[2])    { ammo_left++; }
                                }
     if(flags.enemies[i] != -1) {
         if(actors[flags.enemies[i]].stamina > 0) { e_size++; }
         if(actors[flags.enemies[i]].extra[2])    { ammo_left++; }
                                }
                           }

 if(!f_size) { flags.combat_result = 1; return; }
 if(!e_size) { flags.combat_result = 0; return; }


 // No reason to start until the intro animation has completed
 //
 if(flags.combat_anim < 1.00) { return; }


 // check for all ammo being expended
 //
 if(!ammo_left) {
     flags.combat_timeout++;
     if(flags.combat_timeout >= 200) {
         log(LOG_1, "Team 0 damage = %d", flags.combat_damage[0]);
         log(LOG_1, "Team 1 damage = %d", flags.combat_damage[1]);

         // Use raw damage output to determine winner. In case of a tie,
         // the player loses, because I'm a fucking dick like that, yo.
         // the lesson there is: avoid combat.
         //
         if(flags.combat_damage[0] > flags.combat_damage[1])  { flags.combat_result = 0; }
         if(flags.combat_damage[0] < flags.combat_damage[1])  { flags.combat_result = 1; }
         if(flags.combat_damage[0] == flags.combat_damage[1]) { flags.combat_result = 1; }
                                     }
                }


 rerun:

 for(i=0; i<MAX_CREW; i++) {

     if(!alternate) { crew = flags.friends[i]; }
     else           { crew = flags.enemies[i]; }

     if(crew != -1) {
         if(actors[crew].stamina > 0) {
             switch(actors[crew].extra[0]) {
                 case 0: // In Good Condition
                         actors[crew].extra[1]--;

                         if(actors[crew].extra[1] <= 0) {
                             if(!alternate) {
                                 target = pick_target(1);
                                 if(target != -1) {
                                     fire_shot(crew, actors[crew].gun, actors[crew].pos.x, actors[crew].pos.y, actors[crew].pos.z + 30, target, 1);
                                     delay = guns[actors[crew].gun].delay;
                                     actors[crew].extra[1] = delay + quick_rand(delay);

                                     actors[crew].extra[2]--;
                                     if(actors[crew].extra[2] <= 0) { actors[crew].extra[0] = 9; }
                                                  }
                                            }
                             else {
                                 target = pick_target(0);
                                 if(target != -1) {
                                     fire_shot(crew, actors[crew].gun, actors[crew].pos.x, actors[crew].pos.y, actors[crew].pos.z + 30, target, 0);
                                     delay = guns[actors[crew].gun].delay;
                                     actors[crew].extra[1] = delay + quick_rand(delay);

                                     actors[crew].extra[2]--;
                                     if(actors[crew].extra[2] <= 0) { actors[crew].extra[0] = 9; }
                                                  }
                                  }

                             assign_animation(crew, actors[crew].type + 1, 0);
                                                        }
                         break;

                 case 1: // Stunned
                         actors[crew].extra[2]--;

                         if(actors[crew].extra[2] <= 0) {
                                                        }
                         break;

                 case 9: // out of ammo
                         break;

                                           }
                                      }
                    }
                           }

 if(!end_pass) { end_pass = 1; alternate ^= 1; goto rerun; }

 return;
}





int pick_target(int team)
{
 int t_list[MAX_CREW];
 int _count = 0;
 int target = 0;
 int rnjesus = 0;
 int i = 0;

 for(i=0; i<MAX_CREW; i++) { t_list[i] = -1; }

 for(i=0; i<MAX_CREW; i++) {
     if(!team) {
         if(flags.friends[i] != -1) {
             if(actors[flags.friends[i]].stamina > 0) { t_list[_count] = flags.friends[i]; _count++; }
                                    }
               }
     else {
         if(flags.enemies[i] != -1) {
             if(actors[flags.enemies[i]].stamina > 0) { t_list[_count] = flags.enemies[i]; _count++; }
                                    }
          }
                           }

 if(!_count) { return(-1); }

 rnjesus = quick_rand(_count);
 target = t_list[rnjesus];
 log(LOG_1, "TEAM: %d  - TARGET = %d chosen by rnd %d", team, target, rnjesus);


 return(target);
}




int pick_target_to_heal(int team)
{
 int target  = -1;
 int ceiling = 100;
 int i = 0;

 for(i=0; i<MAX_CREW; i++) {
     if(!team) {
         if(flags.friends[i] != -1) {
             if(actors[flags.friends[i]].stamina > 0 &&
                actors[flags.friends[i]].stamina < ceiling) {
                    ceiling = actors[flags.friends[i]].stamina; target = flags.friends[i];
                                                            }
                                    }
               }
     else {
         if(flags.enemies[i] != -1) {
             if(actors[flags.enemies[i]].stamina > 0 &&
                actors[flags.enemies[i]].stamina < ceiling) {
                    ceiling = actors[flags.enemies[i]].stamina; target = flags.enemies[i];
                                                            }
                                    }
          }
                           }

 log(LOG_1, "TEAM: %d  - TARGET = %d chosen for healing", team, target);
 return(target);
}





void apply_damage(int shot, int target)
{
 // fill in later
 if(shot < 0   || shot >= MAX_ACTORS)   { log(LOG_ALWAYS, "shot OOB in all to %s. Shot = %d", __func__, shot); return;       }
 if(target < 0 || target >= MAX_ACTORS) { log(LOG_ALWAYS, "target OOB in all to %s. Target = %d", __func__, target); return; }

 int sounds[8] = { 75, 73, 74, 76,  75, 74, 73, 75 };
 int ce_x[8] = {  4,  8,  4,  9,   14, 14, 19, 19 };
 int ce_y[8] = {  0,  1,  1,  0,    0,  1,  0,  1 };

 int team = 0;
 float damage     = 0.0;
 float multiplier = 1.0;


 if(actors[target].stamina <= 0) { return; } // nothing to do


 // Calculate special effects for weapon weakness
 //
 switch(guns[actors[shot].gun].effect) {
     case 0: // Vampimp
             if(actors[target].type == 5) {
                 multiplier = 2.0;
                 log(LOG_1, "Vampimp versus Assault Bot = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 6) {
                 multiplier = 2.0;
                 log(LOG_1, "Vampimp versus Inquisitor = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;

     case 1: // Psychocat
             if(actors[target].type == 4) {
                 multiplier = 2.0;
                 log(LOG_1, "Psychocat versus Grunt = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 6) {
                 multiplier = 2.0;
                 log(LOG_1, "Psychocat versus Inquisitor = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;

     case 2: // LuvBot
             if(actors[target].type == 5) {
                 multiplier = 1.5;
                 log(LOG_1, "LuvBot versus Assault bot = 1.5x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 6) {
                 multiplier = 2.0;
                 log(LOG_1, "LuvBot versus Inquisitor = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 7) {
                 multiplier = 1.5;
                 log(LOG_1, "LuvBot versus Shield Goon = 1.5x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;

     case 3: // Siren
             break;

     case 4: // Grunt
             if(actors[target].type == 2) {
                 multiplier = 2.0;
                 log(LOG_1, "Grunt versus LuvBot = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 3) {
                 multiplier = 2.0;
                 log(LOG_1, "Grunt versus Siren = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;

     case 5: // Assault Bot
             if(actors[target].type == 1) {
                 multiplier = 2.0;
                 log(LOG_1, "Assault Bot versus Psychocat = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             if(actors[target].type == 3) {
                 multiplier = 2.0;
                 log(LOG_1, "Assault bot versus Siren = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;

     case 6: // Inquisitor
             break;

     case 7: // Shield Goon
             if(actors[target].type == 3) {
                 multiplier = 2.0;
                 log(LOG_1, "Shield Goon versus Siren = 2x damage bonus. Multiplier = %f", multiplier);
                                          }
             break;
                                       }


 // Adjust for shooter strength buff
 //
 if(actors[shot].atk_buff > 0) {
     multiplier *= 1.5;
    log(LOG_1, "Attack buff in effect. Multiplier = %f", multiplier);
                               }


 // Adjust for shooter strength DEBUFF
 //
 if(actors[shot].atk_buff < 0) {
     multiplier *= 0.5;
    log(LOG_1, "Attack DEBUFF in effect. Multiplier = %f", multiplier);
                               }


 // Adjust for target Defense buff
 //
 if(actors[target].def_buff > 0) {
     multiplier *= 0.5;
    log(LOG_1, "Defense buff in effect. Multiplier = %f", multiplier);
                                 }


 // adjust for target Defense DEBUFF
 //
 if(actors[target].def_buff < 0) {
     multiplier *= 1.5;
    log(LOG_1, "Defense DEBUFF in effect. Multiplier = %f", multiplier);
                                 }


 damage = (guns[actors[shot].gun].damage * multiplier);
 actors[target].stamina -= damage;


 // Record damage for battle decision
 //
 team = actors[target].team ^ 1;
 flags.combat_damage[team] += damage;


 // Set death animation if actor is toasty
 //
 if(actors[target].stamina <= 0 && actors[target].state != 4) {
     // record kill
     if(!actors[target].team) { flags.total_deaths++; }

     actors[target].stamina = 0;
     actors[target].state = 4;
     actors[target].flicker = 1;
     actors[target].cel_x = ce_x[actors[target].type];
     actors[target].cel_y = ce_y[actors[target].type];

     load_coffin(actors[target].owner, actors[target].type);
     kill_animation(target);

     PlaySound(sfx[sounds[actors[target].type]]);
                                                              }

 return;
}





// NOTE: Combat is over, and we have to figure out how to handle the aftermath
//
void resolve_combat()
{
 bool resolve = 0;
 int rnjesus  = 0;
 int _count   = 0;
 int i        = 0;
 int speech   = 0;

 int sp_win[7]  = { 225, 229, 230, 231, 235, 236, 237 };
 int sp_lose[7] = { 222, 232, 233, 234, 238, 239, 240 };

 flags.in_combat++;

 if(flags.in_combat == 40) {
     switch(flags.combat_result) {
         case 0: // Player wins
                 speech = sp_win[quick_rand(7)];

                 // Modifier: 2 or more catgirls in friendly party
                 //
                 if(ships[0].crew[1] > 1) {
                     rnjesus = quick_rand(100);
                     if(rnjesus < 25) { speech = 221; }
                                          }
                 PlaySound(sfx[speech]);

                 for(i=0; i<MAX_ACTORS; i++) {
                     if(actors[i].enabled) {
                         if(actors[i].owner == 0) {
                             assign_animation(i, 11 + actors[i].type, 6);
                             actors[i].anim_timing = 0 - quick_rand(16);
                                                  }
                                           }
                                             }
                 break;

         case 1: // Enemy Wins
                 speech = sp_lose[quick_rand(7)];

                 // Modifier: 2 or more robots in enemy party
                 //
                 if(ships[flags.combat_pending].crew[5] > 1) {
                     rnjesus = quick_rand(100);
                     if(rnjesus < 25) { speech = 223; }
                                                             }
                 PlaySound(sfx[speech]);
                 break;
                                 }
                           }

 if(flags.in_combat > 200) { // was 120
     flags.combat_anim -= 0.05;
     if(flags.combat_anim < 0.0) {
         flags.combat_anim = 0.0;
         resolve = 1;
                                 }
                           }

 if(resolve) {
     // any surviving actors on both ships need to turn invisible
     //
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled) {
             if(actors[i].owner == 0 ||
                actors[i].owner == flags.combat_pending) { actors[i].visible = 0; actors[i].state = 0; }
                               }
                                 }

     // force game over or retreat flags
     //
     switch(flags.combat_result) {
         case 0: // Player wins
                 _count = count_crew(flags.combat_pending);
                 flags.total_wins++;
                 log(LOG_1, "Remaining count for ship %d = %d", flags.combat_pending, _count);
                 if(!_count) {
                     log(LOG_1, "RESULT: ENEMY DESTROYED");
                     ships[flags.combat_pending].destruct = 1;
                             }
                 else {
                     log(LOG_1, "RESULT: ENEMY RETREAT");
                     flags.retreat_pending = flags.combat_pending;

                     flags.wanted_level = 16;
                     if(flags.wanted_anim < 0.001) { flags.wanted_anim = 0.002; }
                      }
                 break;

         case 1: // Enemy Wins
                 _count = count_crew(0);
                 flags.total_losses++;
                 log(LOG_1, "Remaining count for player ship %d = %d", flags.combat_pending, _count);
                 if(!_count) { flags.game_over = 1; log(LOG_1, "RESULT: GAME OVER"); }
                 else {
                     log(LOG_1, "RESULT: PLAYER RETREAT");
                     flags.retreat_pending = 0;

                     flags.wanted_level = 16;
                     if(flags.wanted_anim < 0.001) { flags.wanted_anim = 0.002; }
                      }
                 break;
                                 }

     flags.combat_result    = -1;
     flags.combat_pending   = -1;
     flags.combat_timeout   = 0;
     flags.combat_damage[0] = 0;
     flags.combat_damage[1] = 0;
     flags.in_combat        = 0;

     // Crossfade music
     //
     fade_music_out(1);
     fade_music_in(0);
             }


 return;
}




int clone_celestial_fire(int ship)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(-1); }

 int who = -1;
 int orbit = ships[ship].orbiting;

 who = spawn_actor(0, nodes[orbit].pos.x, nodes[orbit].pos.y + 86, 0.0, 0, 0, 0, 0, 0);

 if(who != -1) {
     actors[who].state   = 8;
     actors[who].effect  = 8;
     actors[who].cel     = tex_32x;
     actors[who].cel_x   = 0;
     actors[who].roll    = 0.4;
     actors[who].angle   = ships[ship].orbit_anim;
     actors[who]._size_w = 173;
     actors[who]._size_h = 173;
     actors[who].scale   = 1.0;
               }

 clear_ship(ship);

 return(who);
}




// NOTE: Creates an actor for a ship, and destroys it.
//       Only necessary because I didn't bother making ships actors.
//       I only did that to see what effect it would have on the system
//       as a whole. I don't feel like it was a net positive.
//
//       Lesson learned :]  Programmers should experiment,
//       and challenge their assumptions from time to time. Surprises happen.
//
// NOTE: [MODE] is 0 for cloning for a kill
//       [MODE] is 1 for cloning for a warp trail
//
int clone_ship(int ship, int mode)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(-1); }

 int i   = 0;
 int who = -1;

 if(!mode) {
     who = spawn_actor(0, ships[ship].last_pos.x, ships[ship].last_pos.y + 48, 0.0, 6, 0, 0, 0, 0);
     if(who != -1) {
         actors[who].state   = 6;
         actors[who].roll    = -3.00 + ((float)quick_rand(600) / 100.0);
         actors[who].v_angle = quick_rand(360);
         actors[who].v_speed = 1.0;

         actors[who].cel     = tex_32x;
         actors[who].cel_x   = ships[ship].cel_x;
         actors[who]._size_w = 32;
         actors[who]._size_h = 32;
         actors[who].scale   = 3.0;
                   }

     // If ship has coffins loaded, eject them all at once...
     // because we're killing off the ship (no chance to dribble out)
     //
     for(i=0; i<MAX_CREW; i++) { if(ships[ship].coffins[0] != -1) { unload_coffin(ship); } }

     clear_ship(ship);
           }

 else {
     who = spawn_actor(0, ships[ship].pos.x, ships[ship].pos.y + 48, 0.0, 6, 0, 0, 0, 0);
     if(who != -1) {
         actors[who].state   = 1;
         actors[who].cel     = tex_32x;
         actors[who].cel_x   = ships[ship].cel_x;
         actors[who]._size_w = 32;
         actors[who]._size_h = 32;
         actors[who].scale   = 3.0;

         if(ships[ship].team == 20) { // Voidstar
         actors[who].effect  = 11;
         actors[who]._size_w = 128;
         actors[who]._size_h = 128;
         actors[who].scale   = 1.0;
         actors[who].angle   = ships[ship].orbit_anim;
                                    }

         if(ships[ship].team == 21) { // Warp coil
         actors[who].effect  = 12;
         actors[who]._size_w = 64;
         actors[who]._size_h = 64;
         actors[who].scale   = 2.0;
         actors[who].angle   = 0.0;//ships[ship].orbit_anim;
                                    }
                   }
      }


 return(who);
}





void warp_ship(int ship)
{
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }

 int done     = 0;
 int cur_node = ships[0].orbiting;
 int _node    = get_empty_node(0);

 if(_node == -1) {
     do { _node = quick_rand(MAX_NODES); } while(_node == cur_node);
                 }


 // Execute draw loop until we reach our destination node
 //
 ships[ship].going_to = _node;
 ships[ship].origin.x = ships[ship].last_pos.x;
 ships[ship].origin.y = ships[ship].last_pos.y;
 ships[ship].orbiting = -1;


 flags.round_anim = 0.0;
 flags.slow_advance = 1;

 do {
     advance_a_ship(ship);
     clone_ship(ship, 1);
     update_and_render();

     if(flags.round_anim >= 1.00) { done = 1; }

    } while(!done);


 // Ships should orbit their destinations
 //
 flags.slow_advance = 0;
 ships[ship].orbiting = _node;
 ships[ship].skip_turn = 0;
 ships[ship].going_to = -1;

 return;
}



// NOTE: Warps a random ship on [TEAM], and locks the camera
//       onto them while they move to their new location
//
void warp_ship_type_event(int team, int notice)
{
 int i              = 0;
 int every_other    = 0;
 int _node          = 0;
 int done           = 0;
 int total          = 0;
 int rando          = 0;
 int rnjesus        = 0;
 int _list[MAX_SHIPS];

 // Find all ships of [TEAM] and catalog them
 //
 for(i=0; i<MAX_SHIPS; i++) { _list[i] = -1; }

 for(i=0; i<MAX_SHIPS; i++) {
     if(ships[i].enabled) {
         if(ships[i].team == team) { _list[total] = i; total++; }
                          }
                            }

 rnjesus = quick_rand(total);
 rando   = _list[rnjesus];

 if(rando < 0 || rando >= MAX_SHIPS) { log(LOG_ALWAYS, "Rando OOB in call to %s. Rando = %d", __func__, rando); return; }


 _node = get_empty_node(1);

 if(_node != -1) {

     if(team == 20 || team == 21) { nodes[ships[rando].orbiting].role = 0; }

     ships[rando].going_to = _node;
     ships[rando].origin.x = ships[rando].last_pos.x;
     ships[rando].origin.y = ships[rando].last_pos.y;
     ships[rando].orbiting = -1;
                 }


 // Advance ships over time
 //
 flags.round_anim   = 0.0;
 flags.slow_advance = 1;
 flags.follow_ship  = rando;
 flags.show_notice  = notice;

 do {
     if(!every_other) { clone_ship(rando, 1); }
     every_other ^= 1;
     advance_a_ship(rando);
     flags.notice_anim = flags.round_anim;

     update_and_render();

     if(flags.round_anim >= 1.00) { done = 1; }

    } while(!done);


 // Lock ships into their new orbits
 //
 flags.slow_advance = 0;

 ships[rando].orbiting  = ships[rando].going_to;
 ships[rando].skip_turn = 0;
 ships[rando].going_to  = -1;

 if(team == 20 || team == 21) { nodes[ships[rando].orbiting].role = team; }


 // Restore original cam values
 //
 flags.follow_ship = 0;
 flags.show_notice = 0;
 return;
}





// NOTE: Warps a random ship on [TEAM], and locks the camera
//       onto them while they move to their new location
//
// NOTE: ** IT IS ASSUMED THIS WILL BE USED FOR FUZZ ONLY **
//       This will not clear out node roles. Be careful
//
void remove_ship_event(int ship, int notice)
{
 int i              = 0;
 int every_other    = 0;
 int _node          = 0;
 int done           = 0;
 int rnjesus        = 0;
 float _angle       = 0.0;
 Vector2 new_pos    = { 0 };

 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }


 PlaySound(sfx[227]); // "Watch out for the fuzz, man"

 // pick random direction to travel
 //
 _angle = quick_rand_f(360.0);


 // Advance ship over time
 //
 flags.round_anim   = 0.0;
 flags.slow_advance = 1;
 flags.follow_ship  = ship;
 flags.show_notice  = notice;

 ships[ship].orbiting = -1;

 do {
     if(!every_other) { clone_ship(ship, 1); }
     every_other ^= 1;

     new_pos = advance(_angle, 10.0);
     ships[ship].pos.x += new_pos.x;
     ships[ship].pos.y += new_pos.y;

     flags.notice_anim = flags.round_anim;

     update_and_render();

     if(flags.round_anim >= 1.00) { done = 1; }

    } while(!done);


 // Destroy ship
 //
 flags.slow_advance = 0;
 clear_ship(ship);


 // Restore original cam values
 //
 flags.follow_ship = 0;
 flags.show_notice = 0;

 return;
}






void heal_crew(int ship, int heal_factor)
{
 int i = 0;

 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled) {
         if(actors[i].owner == ship) {
             if(actors[i].type >= 0 && actors[i].stamina > 0) {
                 actors[i].stamina += heal_factor;
                 if(actors[i].stamina > 100) { actors[i].stamina = 100; }
                                                              }
                                     }
                           }
                             }

 return;
}





void winning_menu()
{
 int i          = 0;
 int slot       = 29;
 int exit       = 0;
 int event      = 0;


 flags.in_menu  = slot;

 // Activate menu and hold until it's open
 //
 menus[slot].portrait    = 29;
 open_menu(slot);
 PlaySound(sfx[85]);

 do {
     update_and_render();
    } while(menus[slot].anim < 1.00);


 do {
     event = get_menu_event(slot);

     switch(event) {
         case 0: log(LOG_1, "User ACK'd their win");
                 exit = 1;
                 break;
                   }

    } while(!exit);


 // Close out menu
 //
 close_menu(slot);
 PlaySound(sfx[84]);
 do {
     update_and_render();
    } while(menus[slot].anim > 0.00);

 flags.in_menu = -1;

 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);
 spawn_chain(4, menus[slot].pos, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y }, (Vector2){menus[slot].pos.x + 480, menus[slot].pos.y + 480}, (Vector2){menus[slot].pos.x, menus[slot].pos.y + 480 }, (Vector2){ 0 }, (Vector2){ 0 }, menus[slot].band_color);

 return;
}




/// NOTE: Might want to make a generic LIFO_QUEUE routine set,
///       and provide this as an optional feature.
///
void load_coffin(int ship, int crew_type)
{
 int i = 0;
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }


 log(LOG_1, "loading coffin in ship %d with crew type %d", ship, crew_type);

 // Push LIFO queue
 //
 for(i=MAX_CREW-1; i>0; i--) { ships[ship].coffins[i] = ships[ship].coffins[i-1]; }

 ships[ship].coffins[0] = crew_type;


 for(i=0; i<MAX_CREW; i++) { log(LOG_1, "coffin queue[%d] = %d", i, ships[ship].coffins[i]); }

 return;
}




void unload_coffin(int ship)
{
 int i = 0;
 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return; }


 // Pop LIFO queue
 //
 spawn_body(ship, ships[ship].coffins[0]);

 for(i=0; i<MAX_CREW-1; i++) { ships[ship].coffins[i] = ships[ship].coffins[i+1]; }

 ships[ship].coffins[MAX_CREW-1] = -1;


 return;
}


