#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "raylib.h"
#include "fastprng.h"

#ifdef USE_ARMCADIA
    #include "ARMCADIA.H"
#endif


#if defined(PLATFORM_WEB)
    #include <emscripten/emscripten.h>
#endif


/// CORE FEATURES TO COMPILE/LINK
///
#define USE_MATH                                   // General purpose math routines
#define USE_UTILITY                                // General purpose routines
#define USE_ANIMATIONS                             // Basic sprite animation routines
#define USE_RINGS                                  // Ring animations
#define USE_EJECTA                                 // Ejecta animations
#define USE_MENUS                                  // Multi-function vpad+mouse supported menus
#define USE_BUTTONS                                // Clickable button support (mouse only for now)
#define USE_TEXTBOX                                // Textbox support
//#define USE_HIGH_SCORE                           // Basic High-score support
#define USE_TIPS                                   // Tutorial/Tooltip support (requires textbox)
//#define USE_BULLETIN                             // Scrolling bulletin at top/bottom screen
#define USE_LOG                                    // Logfile and console echo support
#define USE_FM_SYNTH                               // SFXR sound Rendering support
#define USE_MUSIC                                  // Music stream support via Raylib
//#define USE_RTMIDI                               // MIDI controller support
//#define USE_ARMCADIA                             // Use Armcadia for dynamic tentacles



/// SHADER FEATURES TO COMPILE/LINK
///
//#define USE_SCROLL_SHADER
//#define USE_BLOOM_SHADER
#define USE_WAVE_SHADER


/// PROJECT-CONFIG
///
#define WINDOW_TITLE    "VAMPIRE PIMPS FROM THE WILD BEYOND v3.00"
#define SCREEN_W               1280
#define SCREEN_H               1024
#define GLSL_VERSION            330
#define OS_SCALING              1.0
#define BASE_TICKS               60
#define PIX                     4.0

//#define SKIP_INTRO                               // Skips main menu and goes directly to gameplay loop
//#define ENABLE_DEBUG_KEYS                        // Enables extended F keys
#define NO_ESCAPE_TO_OS                            // Pressing ESC doesn't close the game (Raylib feature)
//#define HIDE_MOUSE_CURSOR                        // Hides mouse cursor. Will be overruled by MOUSE_IS_PRIMARY
#define MOUSE_IS_PRIMARY                           // Mouse is preferred input device for menus (default is kbd)
//#define LOG_TO_CONSOLE                           // Enables local echo of log entries to console

#define MAX_ACTORS              640                // Maximum actors
#define MAX_ANIMATIONS           32                // Maximum number of animation sets
#define MAX_FRAMES               10                // Maximum number of frames PER animation set
#define MAX_SFX                 256                // Maximum number of sfx
#define MAX_SCORE                 2                // Maximum number of scores to track

#define MAIN_MENU_OPTIONS         3
#define MAIN_MENU_BAR_COLOR    32.0
#define MAIN_MENU_BAR_RANGE    20.0
#define MAIN_MENU_BAR_ALPHA    0.35
#define MAIN_MENU_BAR_SPACING    14
#define MAIN_MENU_BAR_SIZE       15

#define SUMMARY_ITEMS             6
#define SUMMARY_ITEM_SIZE       3.0
#define SUMMARY_ITEM_COLOR     40.0
#define SUMMARY_NUMBER_COLOR   64.0
#define SUMMARY_SCORE_COLOR   185.0
#define SUMMARY_BAR_COLOR
#define SUMMARY_BAR_ALPHA      0.25
#define SUMMARY_BAR_SPACING      10
#define SUMMARY_BAR_SIZE         14
#define SUMMARY_HEADER_1_SIZE
#define SUMMARY_HEADER_1_COLOR
#define SUMMARY_HEADER_2_SIZE
#define SUMMARY_HEADER_2_COLOR

#define HUD_LOCATION              0
#define HUD_HEIGHT              140
#define HUD_TEXT_START           10
#define HUD_TEXT_SCALE          3.0
#define HUD_GRADIENT_1         GRAY
#define HUD_GRADIENT_2        BLANK

#define CONTROLS_COLOR_1      WHITE
#define CONTROLS_COLOR_2      BLACK
#define CONTROLS_TEXT_SCALE     3.0
#define CONTROLS_TEXT_START     9.0
#define CONTROLS_TEXT_SPACING   3.0


/// STOCK COLORS
///
#define PURE_GREEN       (Color){   0, 255,   0, 255 }
#define PURE_BLUE        (Color){   0,   0, 255, 255 }
#define PURE_YELLOW      (Color){ 255, 255,   0, 255 }

#define SCORE_PINK       (Color){ 242, 103, 119, 255 }
#define SCORE_ORANGE     (Color){ 255, 161,   0, 255 }
#define SCORE_GREEN      (Color){ 145, 240,   0, 255 }

#define WATER            (Color){  25, 113, 180, 255 }
#define SKY_BLUE         (Color){  25, 113, 180, 255 }
#define EYE_ORANGE       (Color){ 215, 115,   0, 255 }
#define CHILL_CYAN       (Color){  22, 230, 255, 255 }
#define TITLE_GREEN      (Color){ 205, 218,  50, 255 }
#define SKILL_PINK       (Color){ 235, 123, 217, 255 }
#define SHOCK_YELLOW     (Color){ 255, 197,  20, 255 }
#define SHOCK_RED        (Color){ 255,  41,  18, 255 }

#define DARK_GRAY        (Color){  64,  64,  64, 255 }
#define MEDIUM_GRAY      (Color){ 128, 128, 128, 255 }
#define LIGHT_GRAY       (Color){ 192, 192, 192, 255 }

/// PROJECT COLORS
///
#define MENU_ROSE        (Color){ 193, 109, 108, 255 }
#define MENU_VIOLET      (Color){ 140,  87, 175, 255 }
#define MENU_PEACH       (Color){ 209, 140,  71, 255 }

#define CREW_ORANGE      (Color){ 255, 140, 001, 255 }
#define CREW_BLUE        (Color){ 000, 147, 231, 255 }
#define CREW_GREEN       (Color){ 171, 214, 000, 255 }
#define CREW_PURPLE      (Color){ 234, 063, 255, 255 }

#define CARGO_RED        (Color){ 246, 101, 125, 255 }
#define CARGO_YELLOW     (Color){ 255, 208, 047, 255 }
#define CARGO_CYAN       (Color){  95, 238, 216, 255 }
#define CARGO_VIOLET     (Color){ 116,  92, 237, 255 }

#define VOID_PINK        (Color){ 215,  66, 255, 255 }
#define WARP_GREEN       (Color){  36, 183, 151, 255 }
#define SALVAGE_PURPLE   (Color){ 124,  82, 111, 255 }
#define TOOLTIP_BLUE     (Color){  93, 138, 180, 255 }

/// CONVENIENCES
///
#define ARRAY_LEN(...) (sizeof(__VA_ARGS__) / sizeof(*(__VA_ARGS__)))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define TO_DEGREES(a) (a * (180.00 / PI))
#define TO_RADIANS(a) (a * (PI / 180.00))
#define UP                      0
#define DOWN                    1
#define LEFT                    2
#define RIGHT                   3


// This is a little janky but it reduces the need for a big external
// declaration file. Place the PULL_EXTERNS definition exactly
// once in a single source file (for simplicity I use the main file)
// and it'll include the header file and declare an instance
// of everything in here
//
// All other files will see this header as full of externs.
// Only downside is you won't be able to declare initial values
// in here.
//
#ifdef PULL_EXTERNS
    #define EXTERN
#else
    #define EXTERN extern
#endif



/// PROJECT_SPECIFIC
///
#define MAP_SIZE                 4
#define STAR_DENSITY          1280
#define MOVE_SCALE            4.00
#define TILE_SIZE            16.00
#define ACTOR_SCALE           4.00
#define GRAVITY_PULL          0.10
#define INFINITE             99999

#define MAX_CREW_LEVEL           6
#define MAX_CARGO_LEVEL         20
#define MAX_SCANNER_LEVEL        2
#define MAX_FOOD_LEVEL          10
#define MAX_ENGINE_LEVEL         1



/// Standard Assets
///
EXTERN Texture2D tex_8x;
EXTERN Texture2D tex_32x;
EXTERN Texture2D tex_64x;
EXTERN Texture2D box_tex;

EXTERN Texture2D tex_title;
EXTERN Texture2D tex_title_bg;
EXTERN Texture2D tex_summary_bg;

#ifdef MOUSE_IS_PRIMARY
    EXTERN Texture2D tex_mouse_pointer;
#endif

EXTERN Font hi_score_font;
EXTERN Font summary_font;
EXTERN Font controls_font;
EXTERN Font title_font;
EXTERN Font hud_font;

EXTERN Font font1;                               // NoOP font
EXTERN Font font2;                               // 5x5 font
EXTERN Font font3;                               // 3x5 font

EXTERN RenderTexture2D screenbuf;                // For full-screen shader handling
EXTERN RenderTexture2D post_target;              // Secondary FBO used for post-processing

EXTERN fastPRNG::fastXS64 fastR;                 // fastPRNG, because Raylib's is not so good

EXTERN int sfx_buffer[16][2];                    // Allows buffering of sounds (up to 16, by clock ticks)
EXTERN int summary_data[16];                     // Holds copy of summary data for the end game screen

extern const char *main_options[];
extern const char *summary_headers[];
extern const char *summary_items[];
extern const char *summary_units[];

extern const char *menu_keyboard_tips[];
extern const char *menu_gamepad_tips[];
extern const char *game_keyboard_tips[];
extern const char *game_gamepad_tips[];


/// PROJECT-SPECIFIC ASSETS
///
EXTERN Texture2D tex_stars;
EXTERN Texture2D tex_aura;
EXTERN Texture2D tex_l_stripe;
EXTERN Texture2D tex_stripes;
EXTERN Texture2D tex_star;
EXTERN Texture2D tex_voidstar;
EXTERN Texture2D tex_celestial_r;
EXTERN Texture2D tex_celestial_b;
EXTERN Texture2D tex_crew;
EXTERN Texture2D tex_port[48];

EXTERN Image map_1;
EXTERN Image map_2;
EXTERN Image map_3;

EXTERN Sound sfx[MAX_SFX];

EXTERN char custom_text[80];
EXTERN bool hotkey_toggles[512];

EXTERN Vector4 stars[4][STAR_DENSITY];
EXTERN Vector2 star_vars[4][STAR_DENSITY];

extern const char *cargo_names[];
extern const char *planet_names[];
extern const char *menu_options[];
extern const char *menu_titles[];
extern const char *fuzz_titles[];
extern const char *fuzz_names[];


/// Declarations that require initialization
///
extern float anims[16];

extern int puppet;                               // Actor being controlled by the player
extern Camera2D camera;
extern Vector2 mouse;
extern Vector2 mouse_cam;

#ifdef USE_SCROLL_SHADER
    extern Shader scroll;
    extern float scroll_vars[4];                     // Two sets of x/y offsets
    extern int off_xLoc;
    extern int off_yLoc;
#endif

#ifdef USE_BLOOM_SHADER
    extern Shader bloom;
    extern float b_var;
    extern int blo_sLoc;
#endif

extern Shader mosaic;
extern float m_var;
extern int mos_mLoc;
extern int mos_aLoc;

#ifdef USE_WAVE_SHADER
    extern Shader wave;
    extern float s_vars[6];                          // Brush sine wave
    extern float l_vars[6];                          // Title emoji
    extern float k_vars[6];                          // Kelp sine wave
    extern float h_vars[6];                          // Hook sine wave
    extern float f_vars[6];                          // Fire sine wave
    extern int secondsLoc;
    extern int freqXLoc;
    extern int freqYLoc;
    extern int ampXLoc;
    extern int ampYLoc;
    extern int speedXLoc;
    extern int speedYLoc;

    extern int loc_time;
    extern int loc_strength;
    extern float seconds;
    extern float strength;
#endif

extern Shader crt;
extern Shader edge_fade;





/// STANDARD ROUTINES (MAIN)
///
void update_and_render();
void init_game_variables();
void init_tutorial_variables();


/// STANDARD ROUTINES (GAME_LOOP)
///
void core_game_loop();
void draw_control_hints(int mode);
void draw_title_menu(const char *msg);
void draw_title();
void draw_hud();
void draw_high_score();
void draw_summary(int mode);
void draw_project_specific_items();
void end_game(int mode);


/// STANDARD ROUTINES (GAME_UPDATE)
///
void check_delta_updates();
void check_time();
void check_score();
void check_shaders();
void check_animations();
void check_actors();
void check_game_over();
void check_input();
void check_game_specific_anims();
void check_game_specific_logic();
void check_actors_while_paused();
void check_misc_logic();
void check_summary();


/// STANDARD ROUTINES (SPAWNS)
///
void spawn_permanent_actors();
int  spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z, bool force_reverse);
int  spawn_player(int x, int y, int z);
int  spawn_aura(int target, int frame_x, float scale, Color col);



#define MAX_GUNS   8

struct gun_struct
{
 int damage;
 int delay;
 int effect;
 int cel_x;
 int sfx;
 int ammo;

 float velocity;
 Color col;
};

EXTERN gun_struct guns[MAX_GUNS];




#define MAX_SHIPS 32
#define MAX_CREW   8
#define MAX_CARGO  4

struct ship_struct
{
 bool enabled;                // TRUE if this ship is being used
 bool named;                  // TRUE if this ship has a name
 bool skip_turn;              // TRUE if this ship isn't moving this turn
 bool destruct;               // TRUE if this ship is scheduled for destruction

 Vector2 pos;                 // X,Y position in the game map
 Vector2 origin;              // X,Y position of origin before changing nodes
 Vector2 last_pos;            // X,Y of last screen position before changing nodes

 int bribed;                  // # of turns left for ship to ignore player
 int level;                   // Level of ship (only applies to enemies)
 int crew_count;              // Total crew of all types (helper var)
 int crew[MAX_CREW];          // Crew count, per type
 int coffins[MAX_CREW];       // Dead crew waiting to be ejected
 int cargo[MAX_CARGO];        // Cargo count, per type
 int going_to;                // Node the ship is immediately en route to (else -1)
 int orbiting;                // > 0 if orbiting a node
 int c_anim;                  // Coffin ejection delay variable
 int stamina;                 // Lifetime of this ship (despawns when at 0)
 float orbit_anim;            // angle for orbiting animation
 int team;                    // 0 = friendly / 1 = enemy / 2 = other
 int cel_x;                   // Animation frame
 char name[40];               // Name of this ship
};

EXTERN ship_struct ships[MAX_SHIPS];






// Project-specific flags. Can wipe between projects (keep vars at top though)
//
struct flags_struct
{
 bool debug;                      // Enables debugging overlay
 bool tutorial;                   // TRUE if game is in tutorial mode
 bool basic_tutorial;             // TRUE if basic "panel-style" tutorial will be used
 bool tutorial_seen;              // TRUE if basic tutorial has been seen already
 bool game_over;                  // TRUE if "game over" conditions are met
 bool game_ending;                // TRUE if game ending conditions are met (animation before game over is set)
 bool stage_complete;             // TRUE if "stage complete" conditions are met
 bool mouse_menu_check;           // TRUE if we are checking for a mouse click event with a menu
 bool paused;                     // TRUE if game logic is paused
 bool use_crt;                    // TRUE if CRT filter is enabled

 int total_actors;                // Total number of actors per-frame (debug var)
 int gamepad;                     // 1 if a gamepad is selected for primary use (else 0 for keyboard)
 int menu_pos;                    // Selection within menu (or -1 for nothing under mouse cursor)
 int button_pos;                  // Button selected       (or -1 for nothing under mouse cursor)
 int total_options;               // ???
 int game_mode;                   // 0 = title.    1 = Tutorial.     2 = game.    3 = Game Over.     4 = Credits?
 int game_stage;                  // Game stage (if game features multiple stages)
 int selected;                    // Value of player selection (if required)
 float aspect_ratio;              // Holds calculated screen aspect ratio
 float screenSize[2];             // Holds reported window size for passing to shaders
 float zoom_target;               // Target zoom level (will error-correct over time to set camera zoom)

 float max_cam_w;                 // For camera boundary clipping
 float max_cam_h;
 float min_cam_w;
 float min_cam_h;

 float embers[64][3];             // holds data for burning embers over sacrifice gauge

 Color tint_col;                  // Full-screen tint control
 float tint_str;                  //
 float tint_velocity;             //

 float game_speed;                // Holds update rate for game (1.0 = normal, 2.0 = 2x speed, etc)
 float game_velocity;             //

 float bloom;                     // Bloom shader strength (default value is 1.50)

 float mosaic;                    // Mosaic strength control (0-1f)
 float mosaic_velocity;           //
 float mosaic_chirp;              // > 0 if we just need a little blip (OVERRIDES ABOVE VARS)

 int quake_duration;              // Quake frames left to animation

 int show_signal;                 // Triggers a banner across screen for x ticks (-1 to disable)
 int signal_time;                 // How many ticks signal has left on-screen (current)
 int signal_max_time;             // How many ticks signal has left on-screen (maximum)
 int signal_data[2];              // Shows optional data for the signal (i.e. 89723 BONUS!!)
 float signal_f_in;               // Fade in range for signal, based off total time
 float signal_f_out;              // Fade out range for signal, based off total time

 int high_score;                  // High score, if that feature is available
 int score[MAX_SCORE];            // Score
 int score_shake[MAX_SCORE];      // # of frames for score to shake (indicates a problem)
 int score_diff[MAX_SCORE];       // Amount to add into score over time

 float delta;                     // Delta-timing variables
 float delta_factor;              //
 float timing_leftover;           //
 int   last_updates;              //

 float game_time;                 // Elapsed game time
 int   clock_min;                 // Minutes on clock
 int   clock_sec;                 // Seconds on clock
 int   clock_sub;                 // Hundredths on clock

 Vector2 mouse;                   // Mouse coordinates (polled)

 #ifdef USE_RTMIDI
     Color quick_color;           // Holds current quick_color value (if MIDI support is available)
 #endif

 #ifdef USE_LOG
     int log_level;               // Log escalation level (do not set manually. Use pre-defined values)
     FILE *log_handle;            // Handle for logging. Will remain open until session closes
 #endif

 /// Summary items
 int total_rounds;                // Count of rounds played in the game
 int total_spent;                 // Total money spend on items (bribes not included)
 int total_deliveries;            // Total number of employees dropped off
 int total_wins;                  // Total battles won
 int total_losses;                // Total battles lost
 int total_deaths;                // Total crewmates lost in glorious service to their Master Pimps

 /// You can wipe everything below this point
 int show_notice;                 // > 0 if there is a notice on display
 int sky_on_fire;                 // > 0 if there is a celestial fire event active
 int slow_advance;                // > 0 if we're using a slower speed to advance between rounds
 int fuzz_pending;                // != -1 if a fuzz spawn event is pending
 int retreat_pending;             // != -1 if a ship should be forced to retreat
 int combat_damage[2];            // total damage dealt for both friends [0] and enemies [1]
 int combat_result;               // != -1 if combat is resolved (0=friendly victory. 1=enemy victory)
 int combat_pending;              // > -1 if combat with a ship is pending
 int combat_timeout;              // Counter to end the game after the last round has been fired
 int in_combat;                   // > 0 if combat is active, and should be drawn
 int friends[MAX_CREW];           // holds list of friendly actors in a firefight
 int enemies[MAX_CREW];           // holds list of enemy actors in a firefight
 int peace;                       // > 0 if game is in no-combat mode
 int follow_ship;                 // > -1 if laggy cam is turned on (follows a ship, not actor)
 int show_long_haul;              // > 0 if user has selected a node path > 1, and
 int long_haul;                   // > -1 if user has selected a node to auto-pilot to
 int event_pending;               // > 0 if an event is pending
 int turn_pending;                // > 0 if the player has chosen to advance the turn
 int path_locked;                 // > 0 if Node selected, and path solution found!
 int m_hold[2];                   // For tracking repeat mouse clicks
 int node_selected;               // > -1 if a node has been selected by the player
 int last_node;                   // Last node selected (for detecting double clicks)
 int in_menu;                     // > -1 if we're inside a menu, and shouldn't scroll the view
 int salvage_respawn;             // Countdown before salvage can respawn
 int fuzz_respawn;                // Countdown before fuzz can respawn
 int fuzz_level;                  // Difficulty setting (0-3)
 int wanted_level;                // Number of turns before player is in the clear
 int rapid_fire;                  // Counts shots and reduces volume over time
 int shot_ready;                  // Tracks shot readiness (for player)
 int flip_dir;                    // Direction of flip (0 = none, 1 = flipped)
 int flip;                        // Timer for flip stuff

 int retirement;                  // Retirement fund level
 int coffer;                      // Money to spend

 int cargo_wanted;                // Current cargo total that's wanted by planet
 int cargo_total;                 // Current employee level (lol)
 int crew_level;                  // Maximum crew the player can take on
 int cargo_level;                 // Maximum cargo
 int food_level;                  // Maximum food
 int engine_level;                // Current engine upgrade level
 int scanner_level;               // Current scanner level (0-2; 0 = none)
 int food;                        // Current units of food on-hand

 float round_anim;                // 0.0 to 1.0 round animation timer
 float combat_anim;               // 0.0 to 1.0 combat animation sequencer
 float notice_anim;               // 0.0 to 1.0 notice animation sequencer
 float wanted_anim;               // 0.0 to 1.0 wanted animation sequencer
};

EXTERN flags_struct flags;



#ifdef USE_LOG
    #define LOG_ALWAYS  static_cast<int>(1 << 0)
    #define LOG_1       static_cast<int>(1 << 1)
    #define LOG_2       static_cast<int>(1 << 2)
    #define LOG_3       static_cast<int>(1 << 3)
    #define LOG_4       static_cast<int>(1 << 4)

    void open_log();
    void close_log();
    void log(int minimum_priority, const char *parser, ...);
#endif




#ifdef USE_TIPS
    #define MAX_TIPS 32

    struct tip_struct
    {
     bool active;                      // Tip will be evaluated
     bool trigger;                     // tip has been triggered and will display
     int  msg_index;
     Rectangle window;
     Rectangle bounds;
    };

    EXTERN tip_struct tips[MAX_TIPS];

    extern void check_tips();
    extern int spawn_tip_holder(float x, float y, int _tip);
#endif




#ifdef USE_EJECTA
    #define MAX_CHAINS        32
    #define EJECTA_DENSITY    32

    class ejecta_struct
    {
    public:
    Vector2 pos;
    Vector2 path;
    Color col;
    float angle;
    float velocity;
    float strength;
    float mutability;
    float decay_rate;
    };

    class chain_struct
    {
    public:
    bool active;
    ejecta_struct ejecta[EJECTA_DENSITY+1];
    };

    EXTERN chain_struct chain[MAX_CHAINS];


    void set_ejecta(int c_slot, int e_slot, float x, float y, Color col);
    void spawn_chain(int sides, Vector2 one, Vector2 two, Vector2 three, Vector2 four, Vector2 five, Vector2 six, Color col);
    void draw_chains();
    void check_chains();
    void init_chains();
    int  get_free_chain();
#endif




#ifdef USE_RINGS
    #define MAX_RINGS 32

    struct ring_struct
    {
     bool active;
     Vector2 pos;
     float radius;                            // Radius
     float r_change;                          // change in radius over time
     float alpha;                             // alpha (0-1)
     float a_change;                          // Change in alpha over time
     float thickness;                         // Thickness of line
     int segments;                            // # of segments
     int delay;                               // Delay before activation (allows concentric ring spawns at once)
     Color col;
    };

    EXTERN ring_struct rings[MAX_RINGS];

    extern void draw_rings();
    extern void check_rings();
    extern void init_ring(int slot);
    extern int get_free_ring();
    extern int spawn_ring(float x, float y, float radius, float thickness, int delay, Color col);
#endif




#ifdef USE_HIGH_SCORE
    struct hi_struct
    {
     int selected;                // Character currently selected (0-2 valid)
     unsigned char initials[3];   // Stores characters entered by user
     int blink;                   // True if we can display blinking chars
     int blink_timer;             // Timer to track above variable
    };

    EXTERN hi_struct hi_score;
    EXTERN unsigned char high_init[4];

    void write_high_score();
    int  read_high_score();
    void write_32bit_int(FILE *handle, int value);
    int  read_32bit_value(FILE *handle);
    int  get_initial();
#endif



#ifdef USE_BULLETIN
    struct bulletin_struct
    {
     bool enabled;                // Bulletin will be drawn if triggered
     bool position;               // 0 = top of screen   1 = bottom of screen

     int time_left;               // 'Ticks' before bulletin expires
     int height;                  // Height of bulletin in pixels
     int x;                       // X coordinate of scroll position
     int speed;                   // Scroll speed per tick
     int message;                 // Index of message (see bulletins array)

     Font font;                   // Font for bulletin

     float f_scale;               // Scaling value for font
     float f_spacing;             // Spacing value for font

     Color msg_col;               // Color of message
     Color banner_col;            // Color of background
    };

    extern bulletin_struct bulletin;

    extern const char *bulletins[];
    extern void draw_bulletin();
#endif



int spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z, bool force_reverse);
void kill_actor(int slot);




#ifdef USE_MATH
    float scale(float value, float source_min, float source_max, float scaled_min, float scaled_max);
    float tan_scale(float value, float source_min, float source_max, float scaled_min, float scaled_max);
    float log_scale(float value, float source_min, float source_max, float scaled_min, float scaled_max);

    float sin_march(float angle, float magnitude);
    float cos_march(float angle, float magnitude);

    float rescale(float input, float in_min, float in_max, float out_min, float out_max);
    float norm_f(float source, float min, float max);
    float FR_2d_angle_360_f(float x1, float y1, float x2, float y2);
    float degrees_to_radians(float degrees);

    int   FR_2d_dist(int x1, int y1, int x2, int y2);
    int   rotate_to_angle(int angle_a, int angle_b);

    Vector2 rotate_point_2d(float point_x, float point_y, float around_x, float around_y, float angle);
#endif





#ifdef USE_UTILITY
    bool valid_actor(int who);
    int get_free_actor(int mode);
    int get_population(int team);
    int get_radial_population(int who, int radius, int team);
    int is_odd(int number);

    void set_sound(int slot, float pitch, float volume);

    int quick_rand(int range);
    float quick_rand_f(float range);

    int check_floats(float one, float two, float range);
    int check_collision(int who, int target, int radius, int z_adjust);
    bool check_rect_collision(float check_x, float check_y, int x, int y, int w, int h, int diam_x, int diam_y);

    bool compare_color(Color one, Color two);
    Color makeacol(unsigned char r, unsigned char g, unsigned char b, unsigned char a);
    Color grab_gradient(Color c1, Color c2, float banding, float zero_to_one);
    void push_rect(Rectangle *rect, float x, float y, float w, float h);
    void justify_text(const char *text, float y, Font font, Color color, float scale, float spacing);
    void justify_text_xy(const char *text, float x, float y, Font font, Color color, int scale, int spacing);
    void justify_text_r(const char *text, float x, float y, Font font, Color color, int scale, int spacing);
    void justify_text_l(const char *text, float x, float y, Font font, Color color, int scale, int spacing);
    void shadow_text_xy(const char *text, float x, float y, Font font, Color color, Color shadow, int shadow_x, int shadow_y, int scale, int spacing);

    int dist_2d(int x1, int y1, int x2, int y2);
    int get_rough_dir(int x1, int y1, int x2, int y2);
    float get_angle_between(float x1, float y1, float x2, float y2);
    float actor_distance(int actor_1, int actor_2);

    Vector2 advance(float angle, float velocity);

    double easing_cubic(double x);
    float FR_easing_pop_in(float value, float source_min, float source_max, float scaled_min, float scaled_max, float pop_magnitude);

    // NB: These operate on IMAGES, not textures!
    //
    void blit(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);
    void blit_h(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);
    void blit_v(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);

    // NB: These operate on TEXTURES
    //
    void tex_blit(Texture2D tex, int s_x, int s_y, int d_x, int d_y, int w, int h, float rotation, float scale, Color col);
    void tex_blit_f(Texture2D tex, int s_x, int s_y, int d_x, int d_y, int w, int h, float scale, float fade);
    void tex_blit_c(Texture2D tex, int s_x, int s_y, int d_x, int d_y, int w, int h, float scale, Color col);
    void tex_blit_h(Texture2D tex, int s_x, int s_y, int d_x, int d_y, int w, int h, float scale, Color col);
    void DrawTextureTiled(Texture2D texture, Rectangle source, Rectangle dest, Vector2 origin, float rotation, float scale, Color tint);

    void swap_int(int n1, int n2, int index);
    void quicksort(int low, int high);

    void open_comma_file(const char *filename);
    int parse_comma_file(unsigned char *buffer, bool first);

    void trigger_signal(int signal, int max_time, float fade_in_range, float fade_out_range);
#endif




#ifdef USE_ANIMATIONS
    struct anim_struct
    {
     int frame_x[MAX_FRAMES];     // x cell of frame inside sprite sheet (cell, not pixel)
     int frame_y[MAX_FRAMES];     // y cell of frame inside sprite sheet
     int timing[MAX_FRAMES];      // timing in game "ticks" to get to next frame
     int total_frames;            // maximum number of frames in this sequence
    };

    EXTERN anim_struct anim[MAX_ANIMATIONS];

    void generate_animations();
    void assign_animation(int who, int anim, int loops);
    void kill_animation(int who);
#endif




#ifdef USE_RTMIDI
    #include <iostream>
    #include <cstdlib>
    #include "RtMidi.h"

    #define MAX_KNOBS 17

    class device_struct {
        public:
        bool  mode;                                // 0 = Absolute, 1 = Relative
        int   id;                                  // MIDI code for this knob
        float r_min;                               // Raw minimum (input)
        float r_max;                               // Raw maximum (input)
        float s_min;                               // Scaled minimum (output)
        float s_max;                               // Scaled maximum (output)
        float value;                               // Scaled value for use in program
        int   input;                               // Raw value from MIDI controller
                                                   // This value persists if Relative
        float scale;                               // Used *ONLY* by Relative input to scale each tick
                        };

    class FR_midi_struct {
        public:
        bool ignore_sysex;                         // Set these flags to TRUE to ignore this message type
        bool ignore_timing;                        //
        bool ignore_active_sensing;                //

        bool keys[128];                            // Key state (pressed only)
        bool note_active;                          // **Primitive** variable for state of last key command
        int  note_key;                             // **primitive** variable for key # of last key command
        double note_freq;                          // **primitive** variable for frequency of last key command

        device_struct knobs[MAX_KNOBS];
        device_struct pads[MAX_KNOBS];
        int  total_ports;                          // Total MIDI ports, as reported by the lib
        int  current_port;                         // Port we're listening to. Defaults to zero.
        std::vector<unsigned char> message;        // Storage space for messages (later we'll queue this)
        int  msg_bytes;                            // Message byte length
        double msg_stamp;                          // Message timestamp delta
        RtMidiIn *midiin;                          // Obligatory rtMidi construct
                         };

    EXTERN FR_midi_struct midi;
    EXTERN Texture2D tex_knob;
    EXTERN Texture2D tex_knob_plate;

    void init_midi();
    void init_arturia_profile();
    void poll_midi();
    void draw_midi_knobs();
    void update_midi_knob(int slot);
    void set_knob(int slot, float new_value);
    void save_knobs();
    void load_knobs();
    void rescale_midi_device_abs(device_struct &knob, float raw_min, float raw_max, float scale_min, float scale_max);
    void rescale_midi_device_rel(device_struct &knob, float scale_min, float scale_max, float tick_scale);
#endif





#ifdef USE_TEXTBOX
    #define MAX_TEXTBOX          32

    struct textbox_struct
    {
     Rectangle pos;
     float scale;                 // Internally-used window scale variable
     float text_scale;            // Raylib text scale factor
     bool enabled;                // TRUE if this box is active
     bool hold;                   // Holds textbox in open state (can be tied to external logic)
     int viewable_text;           // Number of chars currently exposed (starts at zero, counts up)
     int duration;                // Number of ticks before textbox closes. -1 if disabled
     int mode;                    // 0 = entering, 1 = displaying text, 2 = exiting
     int ticks;                   // Counts time between displaying characters
     const char *msg;
    };

    extern textbox_struct textbox[MAX_TEXTBOX];

    extern float box_tex_x;                // For anchoring happy texture at game over summary
    extern float box_tex_y;                //
    extern int box_update;                 //
    extern const char *textbox_messages[];

    void check_textbox();
    void draw_text_box(int slot);
    void trigger_textbox(int slot, int x, int y, int w, int h, int message, int delay, int duration);
#endif






struct actor_struct
{
 Texture2D cel;         // Pointer to source texture
 Vector3 pos;           // Position
 Vector3 offset;        // Destination (only used to move enemies around)
 Vector3 dest;          // Destination (only used to move enemies around)
 Vector3 path;          // (OPTIONAL) Path of travel (x/y/z)
 Vector3 check;         // (OPTIONAL) Internal variable for checking collisions
 Color color;           // Color tint (WHITE by default)
 Color ref_color;       // Reference Color (if you need to backup original color)

 float blend;           // Blend strength (0-1)    < only applies if not solid >
 float angle;           // Angle of rotation (0-360)
 float v_angle;         // (OPTIONAL) Angle of movement (0-360)
 float v_speed;         // (OPTIONAL) Velocity of movement (tied to v_angle)
 float v_roll;          // (OPTIONAL) != 0 if actor rolls while in flight
 float slew;            // (OPTIONAL) != 0 if actor slides on X axis in flight

 float scale;           // Actor scale (populated during projection)
 float roll;            // != 0 if actor's sprite rolls in place
 float y_shift;         // Dirty hack to physically shift a sprite on y axis by ? pixels (pre-projection)

 float quake;           // > 0 if actor is rolling from damage
 float speed;           // Actor's base speed (0.0 to 0.20) which is added to their movement each turn
 float last_angle;      // Last angle actor had before rotation command (for squid)

 int team;              // 1 = Friendly    2 = Enemy    6 = Inert substances
 int type;              // Per-project sub-type (like a class of enemy or player, for example)
 int shadow;            // > -1 if an actor is their shadow
 int _size_w;           // Dimensions of the cel frame
 int _size_h;           // Dimensions of the cel frame
 int flip;              // Force horizontal mirroring
 int anim_set;          // Animation set to reference
 int anim_step;         // Which step in the anim sequence we're at
 int anim_timing;       // Current step timing for animation
 int anim_loop;         // # of times to loop animation. 0 if only played once
 int mode;              // Used for misc tracking (per-project)
 int home[3];           // Home information (if an origin needs defining)
 int state;             // 0 = inactive. 1-12 = doing stuff
 int effect;            // 0 = none.
 int jitter_x;          // > 0 if actor randomly jitters by 1 PIX +/- on x axis
 int jitter_y;          // > 0 if actor randomly jitters by 1 PIX +/- on y axis
 int owner;             // Actor that owns this (used to define simple relationships)
 int timer;             // Internal timer for animation purposes, or whatever you need
 int stamina;           // Walking and being attacked drain stamina. When <= 0 the actor dies or fades out
 int cel_x;             // X cel within sprite sheet
 int cel_y;             // Y cel within sprite sheet
 int preference;        // Extra data for specifying preferred behavior(s)
 int pref_delay;        // Time before actor can be influenced again
 int locked_to;         // Actor this player is targeting (use owner to keep shot from hitting the shooter)
 int following;         // Actor this actor is following
 int follow_x;          // X Pixel offset for follower
 int follow_y;          // Y Pixel offset for follower
 int extra[4];          // Extra data points for use wherever (lazy!)
 int event;             // > 0 if actor will trigger an event when their commitment has ended
 int inert;             // > 0 if actor should not check collisions. Count down.
 int gun;               // Holds an index into a custom gun struct
 int tip;               // > -1 if actor has a tip attached to them

 int updates;           // Number of movement updates per -tick (default is 1)
 int moved_up;          // > 0 if actor has moved up recently
 int moved_down;        // > 0 if actor has moved down recently
 int moved_wait;        // > 0 if actor needs to wait before heading to next way-point

 bool d_repeat[4];      // Tag for direction repeat preclusion
 bool b_repeat[4];      // Tag for button repeat preclusion
 int d_pad[4];          // Janky virtual gamepad (refine later)
 int buttons[4];        //

 int shader;            // > 0 if a shader is in use with this actor
 float s_vars[10];      // Storage space for shader vars. These will vary according to the shader in use

 bool immortal;         // TRUE is actor is immune to death
 bool decal;            // Indicates actor is a floor decal, and will be drawn before normal actors
 bool flicker;          // Tracks actor flicker (used by effects)
 bool mirror;           // TRUE if actor is mirrored on X-axis
 bool gravity;          // TRUE if gravity affects Y coordinate
 bool launched;         // TRUE if player has been launched (do not repeat action until land)
 bool projected;        // TRUE is actor is passed through projection stage before drawing
 bool enabled;          // TRUE if this actor is active
 bool visible;          // TRUE if this actor will be drawn to screen
 bool accessory;        // TRUE if actor will be rendered with a package object
                        // and should not be drawn or interacted with otherwise

 /// Project-specific additions
 ///
 int atk_buff;          // != 0 if a buff or debuff is applied.
 int def_buff;          // != 0 if a buff or debuff is applied.
};

EXTERN actor_struct actors[MAX_ACTORS];
EXTERN int sorted[MAX_ACTORS][2];




#ifdef USE_MUSIC
    #define MAX_TRACKS 8
    struct music_struct {
      Music track;
      bool  repeat;      // TRUE if we want this track to repeat
      bool  loaded;      // TRUE if file has been loaded
      int   state;       // State of track (0-6)
      float position;    // Position in seconds
      float volume;      // 0.0 - 1.0 (set by music playing/fading handler)
      float max_volume;  // 0.0 - 1.0 (factor that determines maximum volume)
      float fade_rate;   // 0.0 - 1.0 (0.01 is base rate)
      float pitch;       // 1.0 is base pitch
                        };

    EXTERN music_struct muzak[MAX_TRACKS];

    void init_music();
    void load_music(int slot, const char *file);
    void music_handler();
    void start_music(int slot);
    void rewind_music(int slot);
    void fade_all_music();
    void fade_music_in(int slot);
    void resume_music(int slot);
    void fade_music_out(int slot);
#endif




#ifdef USE_FM_SYNTH
    #define fm_PI 3.14159265f
    #define FM_MONO_INPUT                         // Comment this out if STEREO input is necessary

    // Contains FM wave form settings
    //
    class FM_form {
      public:

      bool  filter_on; // unused, but stored in .sfs files
      int   wave_type;
      float base_freq;
      float freq_limit;
      float freq_ramp;
      float freq_dramp;
      float duty;
      float duty_ramp;
      float vib_strength;
      float vib_speed;
      float vib_delay;
      float env_attack;
      float env_sustain;
      float env_decay;
      float env_punch;
      float lpf_resonance;
      float lpf_freq;
      float lpf_ramp;
      float hpf_freq;
      float hpf_ramp;
      float pha_offset;
      float pha_ramp;
      float repeat_speed;
      float arp_speed;
      float arp_mod;
                  };




    // Contains misc FM synth settings
    //
    class FM_synth {
      public:

      float  master_vol;
      float  sound_vol;
      bool   playing_sample;
      int    phase;
      double fperiod;
      double fmaxperiod;
      double fslide;
      double fdslide;
      int    period;
      float  square_duty;
      float  square_slide;
      int    env_stage;
      int    env_time;
      int    env_length[3];
      float  env_vol;
      float  fphase;
      float  fdphase;
      int    iphase;
      float  phaser_buffer[1024];
      int    ipp;
      float  noise_buffer[32];
      float  fltp;
      float  fltdp;
      float  fltw;
      float  fltw_d;
      float  fltdmp;
      float  fltphp;
      float  flthp;
      float  flthp_d;
      float  vib_phase;
      float  vib_speed;
      float  vib_amp;
      int    rep_time;
      int    rep_limit;
      int    arp_time;
      int    arp_limit;
      double arp_mod;
      int    wav_bits;
      int    wav_freq;
      int    file_sampleswritten;
      float  filesample;
      int    fileacc;
                   };

    // From FR_FM_SYNTH -
    //
    extern bool FM_save_form(const char *filename);
    extern bool FM_load_form(const char *filename);
    extern void FM_reset_sample(bool restart);
    extern void FM_load_assets();
    extern void FM_reset_form();
    extern void FM_play_sample();
    extern Sound FM_export_to_Sound();
#endif





/// PROJECT-SPECIFIC DECLARATIONS -------------------------------------------- ///
///
int  find_node(Vector2 mouse_pos);
int  find_path(int ship, int start_node, int dest_node);
int  check_events();
int  examine_path(int node, int linked_node);
void record_path(int ship, int depth);
void traversal(int ship, int current_node, int dest_node);
void clear_path();
void clear_nodes();
void parse_map(Image image);

int  bacon_menu(int enemy_ship);
void winning_menu();
void define_menus();
void define_buttons();
void cargo_menu();
void salvage_menu(int _node);
void rescue_menu(int _node);
void celestial_fire_menu(int _node);
void voidstar_menu();
void warp_coil_menu();
void pimpstation_menu();
void advanced_pimping_menu();
void starship_pimping_menu();
void summary_menu();
void draw_menu_extras(int slot);
void error_bleep();
void add_funds(int amount);
void long_haul();
void clear_path_selection();
void laggy_cam(int target_ship);
void clip_camera();
void zoom_tracking();

void draw_combat();
void prepare_for_combat(int enemy_ship);
void process_combat();
void resolve_combat();
void apply_damage(int shot, int target);
void scan_roster(bool friends_only);
void load_coffin(int ship, int crew_type);
void unload_coffin(int ship);

int  pick_target(int team);
int  pick_target_to_heal(int team);
int  clone_ship(int ship, int mode);
int  clone_celestial_fire(int ship);


int  assign_crew(int ship, int type);
int  count_crew(int ship);
void heal_crew(int ship, int heal_factor);
void create_player_ship(int slot, int force_node);
int  create_fuzz_ship(int enemy_level, int force_node);
void create_pimpstations(int quantity, int force_node);
void create_advanced_stations(int quantity, int force_node);
void create_upgrade_station(int quantity, int force_node);
void create_voidstars();
void create_warp_coil();
void create_misc_event(int type, int force_node);
void clear_ship(int slot);
int  get_free_ship();
int  get_empty_node(bool must_be_vacant);
int  find_event(int _node);

int  spawn_radiance(float x, float y, float z, int target, bool dir);
int  spawn_blot(float x, float y, float z, Color col, float scale_factor, float speed_factor);
int  spawn_body(int ship, int crew_type);

Vector2 get_star_pos(int i, int j, Vector2 off_center);
void draw_warning();
void draw_ship(int _node, int _ship, float abs_x, float abs_y);
void draw_stars();
void draw_nodes();
void draw_node_connections();
void draw_node_name(int what);
void draw_hud_manifest();
void draw_notice();
void next_round(bool pass_turn);
void advance_ships();
void warp_ship(int ship);
void warp_ship_type_event(int team, int notice);
void remove_ship_event(int ship, int notice);
void advance_a_ship(int ship);
void ship_wander(int ship, bool forced);
void force_retreat(int ship);
bool is_game_over();
void fire_shot(int shooter, int gun, float x, float y, float z, int target, int direction);
void fire_effect(int shooter, int gun);
void kill_player();


#define MAX_NODES 128
#define MAX_LINKS   6
#define MAX_HOES   10

struct node_struct
{
 bool mapped;                 // TRUE if this node has been parsed by the map (can be used)
 bool enabled;                // TRUE if this node is being used
 bool named;                  // TRUE if this node has a name

 Vector2 pos;                 // X,Y position in the game map
 int role;                    // 0 = none / 1 or higher = station or other event
 int links[MAX_LINKS];        // Links to other nodes
 int accessory;               // Accessory badge, else 0 for none
 int planet_reload;           // turns left before reloading inventory (+1 hoes)
 int planet_mutate;           // turns left before a mutation
 int planet_cel;              // Planet cel in x64 texture
 int  fire_count;             // Number of celestial fire pulses left
 float fire_radius;           // Radius of celestial fire
 float planet_anim;           // Planets slowly orbit the node
 int anim_state;              // Animation state
 float anim_var;              // Generic animation variable
 float band_hue;              // Random hue (for HSV calc) for cargo menus

 int cargo_offer;             // Type of cargo on offer
 int cargo_count;             // Amount of cargo on offer
 int cargo_pref;              // Type of cargo desired
 int scanned;                 // Gives free scan data for x rounds

 char name[40];               // Name of this node
};

EXTERN node_struct nodes[MAX_NODES];

EXTERN int node_traversed[MAX_NODES];
EXTERN int draw_path[MAX_NODES];           // Copy of player's final path, to be drawn
EXTERN int node_path[MAX_NODES];           // Final path for last ship that called pathfinding
EXTERN int node_queue[MAX_NODES];          // Traversal queue
EXTERN int queue_depth;
EXTERN int queue_distance;                 // Number of steps to be below for shorter path




#ifdef USE_MENUS
    #define MAX_MENUS   32
    #define MAX_OPTIONS 8

    struct menu_struct
    {
     bool enabled;                // TRUE if menu should be tracked
     bool options_only;           // TRUE if no window dressing is drawn
     bool open;                   // TRUE if menu should open (animation)
     bool close;                  // TRUE if menu should close (animation)

     Font title_font;             // Font for title block
     Font opt_font;               // Font for options
     Vector2 pos;                 // Menu position
     Vector2 opt_offset;          // Menu offset from menu position

     Color on_text;               // Option text color when selected
     Color off_text;              // Option text color when not selected
     Color inv_text;              // Option text color when invalid
     Color off_color;             // Selection bar color when not selected
     Color band_color;            // Crossbar color

     float anim;                  // 0-1 external scaled animation tracking var
     float anim_linear;           // 0-1 internal linear tracking var
     int portrait;                // portrait frame (or -1 for none)
     int title;                   // title entry (from string list) (or -1 for none)
     int extras;                  // value for custom drawing routine
     int num_options;             // Number of options available in this menu
     int options[MAX_OPTIONS];    // List of options (pulls entry from string list) (-1 for none)
     int invalid[MAX_OPTIONS];    // List of options slots that are invalid (and should be drawn differently)
     float opt_height;            // Height of entries, in SCREEN_H/x
     float opt_width;             // Width of entries, in SCREEN_W/x
     float opt_spacing;           // Space between entries, in SCREEN_H/x
     float opt_scaling;           // Font scaling

     float opt_color;             // Option selection bar HUE
     float opt_range;             // Option selection bar pulsation range
     float opt_alpha;             // Option bar alpha

     float off_alpha;             // Option bar alpha (when not selected)

     float inv_color;             // Invalid Option selection bar HUE
     float inv_range;             // Invalid Option selection bar pulsation range
     float inv_alpha;             // Invalid Option bar alpha
    };

    EXTERN menu_struct menus[MAX_MENUS];

    void check_menus();
    void open_menu(int slot);
    void close_menu(int slot);
    void draw_menu(int slot);
    void init_menu(int slot);
    int  get_menu_event(int slot);
#endif



#ifdef USE_BUTTONS
    #define MAX_BUTTONS   8

    struct button_struct
    {
     bool enabled;                // TRUE if menu should be tracked
     bool two_line;               // TRUE if button has a two-line descriptor (fix this jank later)
     Font font;                   // Font for text
     float font_scaling;          // Font scaling factor
     float font_spacing;          // Font spacing factor

     Color hover_back;            // Colors for hovering over button
     Color hover_text;
     Color idle_back;             // Colors for when mouse isn't over button
     Color idle_text;
     Vector2 pos;                 // Screen position
     int w;                       // Width
     int h;                       // Height
     char line_1[40];             // Descriptor (line 1)
     char line_2[40];             // Descriptor (line 2)
    };

    EXTERN button_struct buttons[MAX_BUTTONS];

    void draw_button(int slot);
    void init_button(int slot);
    int  check_button_events();

#endif



extern int upgrade_cost[4];
extern int crew_cost[MAX_CREW];
extern int cargo_cost[MAX_CARGO];
