#include "project.h"





// NOTE: Used to initially populate important stuff that'll never change between sessions
//       Puppet is the player actor, and so its virtual gamepad will be tracked for input.
//
void spawn_permanent_actors()
{
 int who = -1;

 // Create player (in this game it's not used, other than a hook for potential vpad input)
 //
 puppet = spawn_player(200, 600, 0);
 if(puppet == -1) { log(LOG_ALWAYS, "Fatal error. Couldn't spawn player. Baborting."); exit(0); }
 actors[puppet].visible = 0;

 return;
}





// Spawns a new actor. Types are 0-x for default skin
//
int spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z, bool force_reverse)
{
 int slot = -1;

 if(!force_reverse) { slot = get_free_actor(0); }
 else               { slot = get_free_actor(1); }


 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { return(-1); }

 actors[slot].mode      = 0;
 actors[slot].visible   = 1;
 actors[slot].effect    = 0;
 actors[slot].projected = 0;
 actors[slot].blend     = 1.0f;
 actors[slot].angle     = 0.0f;
 actors[slot].timer     = 0;
 actors[slot].stamina   = 10000;
 actors[slot].v_angle   = 0.0;
 actors[slot].locked_to = -1;

 actors[slot].pos.x     = x;
 actors[slot].pos.y     = y;
 actors[slot].pos.z     = z;

 actors[slot].path.x    = path_x;
 actors[slot].path.y    = path_y;
 actors[slot].path.z    = path_z;

 actors[slot].cel       = tex_32x; // default value
 actors[slot].cel_x     = cell_x;
 actors[slot].cel_y     = 0;

 actors[slot].state     = 0;
 actors[slot].scale     = 1.0;

 actors[slot].enabled = 1;

 return(slot);
}





int spawn_player(int x, int y, int z)
{
 int who = -1;

 who = spawn_actor(0, 0, 0, 0, 1, 0.0, 0.0, 0.0, 0);

 if(who != -1) {
     actors[who].angle   = 0;
     actors[who].team    = 1;
     actors[who].scale   = 3.0;
     actors[who].state   = 0;
     actors[who].pos.x   = x;
     actors[who].pos.y   = y;
     actors[who].pos.z   = z;
     actors[who].cel     = tex_32x;
     actors[who]._size_w = 32;
     actors[who]._size_h = 32;
     actors[who].stamina = 1000;
     actors[who].color   = WHITE;
     actors[who].enabled = 1;
               }

 return(who);
}





void kill_actor(int slot)
{

 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { log(LOG_ALWAYS, "ERROR: Invalid slot in kill command. Slot = %d", slot); return; }

 int i = 0;

 // If an actor is following us, standard behavior would be to kill it
 // Leave a state open for this (99)
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled && actors[i].following == slot && i != slot) { actors[i].state = 99; }
                             }
 /*
 // Check if tip is active
 //
 if(actors[slot].tip != -1) {
     if(actors[slot].tip >= 0 && actors[slot].tip < MAX_TIPS) {
         if(textbox[actors[slot].tip].hold) { textbox[actors[slot].tip].hold = 0; }
                                                              }
                            }
 */

 actors[slot].enabled     = 0;
 actors[slot].immortal    = 0;
 actors[slot].visible     = 1;
 actors[slot].projected   = 0;
 actors[slot].mirror      = 0;
 actors[slot].team        = 0;
 actors[slot].type        = -1;
 actors[slot].decal       = 0;
 actors[slot].shader      = 0;
 actors[slot].shadow      = 0;
 actors[slot].accessory   = 0;
 actors[slot].gun         = 0;
 actors[slot].y_shift     = 0;
 actors[slot].jitter_x    = 0;
 actors[slot].jitter_y    = 0;
 actors[slot].slew        = 0;
 actors[slot].gravity     = 0;
 actors[slot].launched    = 0;
 actors[slot].flicker     = 0;
 actors[slot].anim_set    = 0;
 actors[slot].anim_step   = 0;
 actors[slot].anim_timing = 0;
 actors[slot].anim_loop   = 0;
 actors[slot].effect      = 0;
 actors[slot].state       = 0;
 actors[slot].mode        = 0;
 actors[slot].flip        = 0;
 actors[slot].timer       = 0;
 actors[slot]._size_w     = 16; // Default value
 actors[slot]._size_h     = 16; // Default value
 actors[slot].inert       = 0;
 actors[slot].extra[0]    = 0;
 actors[slot].extra[1]    = 0;
 actors[slot].extra[2]    = 0;
 actors[slot].extra[3]    = 0;
 actors[slot].home[0]     = -1;
 actors[slot].home[1]     = -1;
 actors[slot].home[2]     = -1;
 actors[slot].preference  = -1;
 actors[slot].pref_delay  = 0;
 actors[slot].event       = 0;
 actors[slot].timer       = 0;
 actors[slot].cel_x       = 0;
 actors[slot].cel_y       = 0;
 actors[slot].tip         = -1;
 actors[slot].following   = -1;
 actors[slot].follow_x    = 0;
 actors[slot].follow_y    = 0;
 actors[slot].owner       = -1;
 actors[slot].locked_to   = -1;
 actors[slot].scale       = 1.0f;
 actors[slot].v_angle     = 0.0f;
 actors[slot].v_roll      = 0.0f;
 actors[slot].v_speed     = 0.0f;
 actors[slot].blend       = 1.0f;
 actors[slot].angle       = 0.0f;
 actors[slot].roll        = 0.0f;
 actors[slot].quake       = 0.0f;
 actors[slot].angle       = 0.0f;
 actors[slot].last_angle  = 0.0f;
 actors[slot].speed       = 1.0f; // Default value
 actors[slot].stamina     = 1000;

 actors[slot].updates     = 1;
 actors[slot].moved_up    = 0;
 actors[slot].moved_down  = 0;
 actors[slot].moved_wait  = 0;

 for(i=0; i<4; i++) {
     actors[slot].d_pad[i]    = 0;
     actors[slot].d_repeat[i] = 0;
     actors[slot].buttons[i]  = 0;
     actors[slot].b_repeat[i] = 0;
                    }

 actors[slot].shader      = 0;
 for(i=0; i<10; i++) { actors[slot].s_vars[i] = 0.0; }

 actors[slot].path.x      = 0;
 actors[slot].path.y      = 0;
 actors[slot].path.z      = 0;

 actors[slot].pos.x       = 0;
 actors[slot].pos.y       = 0;
 actors[slot].pos.z       = 0;

 actors[slot].dest.x      = 0;
 actors[slot].dest.y      = 0;
 actors[slot].dest.z      = 0;

 actors[slot].offset.x    = 0;
 actors[slot].offset.y    = 0;
 actors[slot].offset.z    = 0;

 actors[slot].cel         = tex_32x;
 actors[slot].color       = WHITE;
 actors[slot].ref_color   = WHITE;

 /// Project-specific stuff
 ///
 actors[slot].atk_buff    = 0;
 actors[slot].def_buff    = 0;

 return;
}




/// Project-specific spawns below this point
///
int spawn_body(int ship, int crew_type)
{
 int who = -1;
 Color crewset[8] = { CREW_ORANGE, CREW_BLUE, CREW_GREEN, CREW_PURPLE, CARGO_RED, CARGO_YELLOW, SKILL_PINK, LIGHT_GRAY };

 if(ship < 0 || ship >= MAX_SHIPS) { log(LOG_ALWAYS, "Ship OOB in call to %s. Ship = %d", __func__, ship); return(-1); }
 if(crew_type < 0 || crew_type >= 8) { log(LOG_ALWAYS, "crew OOB in call to %s. Crew = %d", __func__, crew_type); return(-1); }


 who = spawn_actor(0, ships[ship].last_pos.x, ships[ship].last_pos.y, 0.0, 1, 0.0, 0.0, 0.0, 0);

 if(who != -1) {
     actors[who].team    = 3;
     actors[who].scale   = 3.0;

     actors[who].state   = 7;
     actors[who].v_speed = 0.5 + quick_rand_f(0.5);
     actors[who].roll    = -3.00 + ((float)quick_rand(600) / 100.0);

     actors[who].v_angle = quick_rand(360);
     actors[who].cel     = tex_8x;
     actors[who]._size_w = 8;
     actors[who]._size_h = 8;
     actors[who].stamina = 300;
     actors[who].color   = crewset[crew_type];

     actors[who].enabled = 1;
               }

 return(who);
}




int spawn_blot(float x, float y, float z, Color col, float scale_factor, float speed_factor)
{
 int who = -1;

 who = spawn_actor(0, x, y, z, 1, 0.0, 0.0, 0.0, 0);

 if(who != -1) {
     actors[who].team    = 3;
     actors[who].scale   = 1.0 + (float)(((float)quick_rand(2000)) / 1000.0);
     actors[who].scale *= scale_factor;

     actors[who].state   = 3;
     actors[who].effect  = 1;
     actors[who].flicker = 1;
     actors[who].v_speed = 2.0 + (float)quick_rand(300) / 100.0;
     actors[who].v_speed *= speed_factor;

     actors[who].v_angle = quick_rand(360);
     actors[who].cel     = tex_32x;
     actors[who]._size_w = 32;
     actors[who]._size_h = 32;
     actors[who].stamina = 200;
     actors[who].color   = col;

     actors[who].enabled = 1;
               }

 return(who);
}




int spawn_radiance(float x, float y, float z, int target, bool dir)
{
 int who = -1;

 if(target < 0 || target >= MAX_ACTORS) { log(LOG_ALWAYS, "target OOB in all to %s. Target = %d", __func__, target); return(-1); }

 who = spawn_actor(0, x, y, z, 3, 0.0, 0.0, 0.0, 0);

 if(who != -1) {
     actors[who].team    = 1;
     actors[who].scale   = 4.0;
     actors[who].effect  = 3;     // Additive blur
     actors[who].y_shift = 3;
     actors[who].state   = 5;
     actors[who].flicker = 0;
     actors[who].v_speed = 21.0;
     actors[who].cel     = tex_32x;
     actors[who]._size_w = 32;
     actors[who]._size_h = 32;
     actors[who].stamina = 55;
     actors[who].preference = dir;
     actors[who].locked_to  = target;
     actors[who].color      = WHITE;

     if(dir) { // Right shot
         actors[who].v_angle = 90.0;
         actors[who].flip = 1;
             }
     else {    // Left shot
         actors[who].v_angle = 270.0;
          }

     actors[who].enabled = 1;
               }

 return(who);
}



// NOTE: Spawns an aura over the top of an actor (tries to pick actor from top end)
//       if Z-sort isn't in use. Will fade out and rotate
//
int spawn_aura(int target, int frame_x, float scale, Color col)
{
 int who = -1;

 if(target < 0 || target >= MAX_ACTORS) { log(LOG_ALWAYS, "target OOB in all to %s. Target = %d", __func__, target); return(-1); }

 who = spawn_actor(0, actors[target].pos.x, actors[target].pos.y + 10, actors[target].pos.z, frame_x, 0.0, 0.0, 0.0, 1);

 if(who != -1) {
     actors[who].team    = 90;    // Put to some neutral value
     actors[who].scale   = scale;
     actors[who].effect  = 3;
     actors[who].state   = 98;
     actors[who].flicker = 1;
     actors[who].cel     = tex_32x;
     actors[who]._size_w = 32;
     actors[who]._size_h = 32;
     actors[who].stamina = 60;
     actors[who].color   = col;
     actors[who].enabled = 1;
               }

 return(who);
}
